/*
 * File:
 *
 * Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2014 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __ETHERDATA_H
#define __ETHERDATA_H
#include "EtherMESH.h"
#include "EtherDATA.ddl.h"

// 'core' value attribuutes
enum
	{
	// Type           // Enum
	//======================================
	/* byte */			kDbValueType = 1,
	/* byte */			kDbValueSize,
	/* byte */			kDbValueCount,
	/* string */		kDbValueIdent,
	/* void (*)() */	kDbValueGetHandler,
	/* void (*)() */	kDbValueSetHandler,
	/* byte */			kDbValueAttributeCount,
	//
	/* none */			kDbMaxCoreValue
	};

// scan return values
#define kScanContinue 0
#define kScanStop 1

typedef float real;
typedef char *string;

typedef byte dbSize;
typedef unsigned int dbValue;
typedef unsigned char dbIndex;

typedef void (*dbCallback)(string, dbValue, void *value);
typedef STATUS (*dbScanHandler)(string group, dbValue ref, void *cookie, byte index, void *value);

extern string dbName;

// if used, these must be called >before< dbInit() to have useful effect
// NB: all nodes must have coherent config values!!
void setMeshInterval(word interval);			// set the mesh sync interval
void setMeshWindow(word window);					// set mesh synchronization window (in milliseconds, max 128)
void setMeshTimeout(word count);					// set mesh synchronization timeout count (# multiples of sync interval)
void setMeshPendCount(word count);				// set the message pend count

STATUS dbInit(string);
STATUS dbRename(emAddr, string);
STATUS dbRegister(dbValue);

STATUS dbGetType(dbValue ref, byte *type);	// recover the internal value type code

STATUS dbLog(dbValue, dbIndex, word, bool);	// turn on logging for a value
STATUS dbLogStop(dbValue ref);					// stop logging
STATUS dbFlushLog(dbValue, word);				// flush n log entries, -1 for all
STATUS dbScanLog(dbValue ref, void *cookie, dbScanHandler);
															// scan through all log entries
STATUS dbGetAttribute(dbValue, dbIndex, void *);

STATUS dbGetName(dbValue, string *);         // get the name of a value
STATUS dbFindValue(string, dbValue *);			// find value using name

STATUS dbGet(dbValue, void *);               // get singleton value
STATUS dbGetArray(dbValue, dbIndex, void *); // get value[index] of type
STATUS dbScanArray(dbValue ref, void *cookie, dbScanHandler);
														   // scan an array of values

STATUS dbGetValue(string, dbValue, void *, word timeout);
STATUS dbAsyncGetValue(string, dbValue, dbCallback, word timeout);
														   // get a value from specific group/node

STATUS dbGetArrayValue(string, dbValue, dbIndex, void *, word timeout);
STATUS dbAsyncGetArrayValue(string, dbValue, dbIndex, dbCallback, word timeout);
                                             // get an array value[index] from specific group/node
STATUS dbScanArrayFrom(string, dbValue ref, void *cookie, dbScanHandler, word timeout);
STATUS dbScanArrayValue(string, dbValue ref, void *cookie, dbScanHandler, word timeout);
														   // scan an array from specific group/node

STATUS dbSet(dbValue, ...);   					// set singleton value of type
STATUS dbSetArray(dbValue, dbIndex, ...);    // set value[index] of type

STATUS dbSetValue(string, dbValue, ...);
STATUS dbSetValue(string, dbValue, ...);
STATUS dbSetValueByRef(string, dbValue, void *ref);
STATUS dbSetArrayValue(string, dbValue, dbIndex, ...);

// <depreciated>
#define dbGetFrom dbGetValue
#define dbGetArrayFrom dbGetArrayValue
//STATUS dbSetGroupValue(string, dbValue, ...);
//STATUS dbSetGroupValueByRef(string, dbValue, void *ref);
//STATUS dbSetGroupArrayValue(string, dbValue, dbIndex, ...);
// </depreciated>

STATUS dbShiftArray(dbValue, byte);				// shift array value left(-n)/right(+n)

STATUS dbGetName(dbValue, string *);         // get the name of a value
STATUS dbFindValue(string, dbValue *);			// find value using name

// supported encoding schemes
#define kStringEncoding 1

dbSize dbEncode(byte scheme, dbValue ref, dbSize size, byte *buf, dbSize index);
															// encode a value into a buf according to encoding scheme

// async versions

#if 0
STATUS dbAsyncGet(dbValue, dbCallback);               // get singleton value
STATUS dbAsyncGetArray(dbValue, dbIndex, dbCallback); // get value[index] of type
STATUS dbAsyncGetFrom(string, dbValue, dbCallback);   // get a value from specific group/node
STATUS dbAsyncGetArrayFrom(string, dbValue, dbIndex, dbCallback);
	                                            	      // get a value[index] from specific group/node
STATUS dbAsyncSet(dbValue, dbCallback, ...);          // set singleton value of type
STATUS dbAsyncSetArray(dbValue, dbIndex, dbCallback, ...);// set value[index] of type
#endif

STATUS dbPublish(dbValue);										// publish value updates
STATUS dbWithdraw(dbValue);									// withdraw value updates from publication
STATUS dbSubscribe(string, dbValue, dbCallback);		// subscribe to value updates
STATUS dbUnsubscribe(string, dbValue);						// un-subscribe to value updates

#define TYPE_UNDEF     0
#define TYPE_REMOTE    1
#define TYPE_CHAR      2
#define TYPE_INT       3
#define TYPE_WORD      4
#define TYPE_BYTE      5
#define TYPE_WYDE      6
#define TYPE_TETA      7
#define TYPE_REAL      8
#define TYPE_STRINGREF 9
#define TYPE_STRINGBUF 10
#define TYPE_BOOL      11
#define TYPE_INT8      12
#define TYPE_INT16     13
#define TYPE_INT32     14
#define TYPE_UINT8     15
#define TYPE_UINT16    16
#define TYPE_UINT32    17
#define TYPE_GROUP     18
#define TYPE_ACCESS    19

char *sysGetEnv(const char *);

#endif//__ETHERDATA_H
