/*
 *	File:	Koliada - Koliada.h
 *
 *	Contains: KoliadaES header
 *
 * Koliada - goddess of time and the winter solstice
 *
 * Kiss - Keep It Small & Simple = reduce complexity, system footprint & bugs!
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#include <stdlib.h>

#if 0
#if M4_MACROS
	#file __file__
	include(debug.h)
#endif
#line 31
#endif
#ifndef	__KOLIADA_H
#define	__KOLIADA_H
#define KOLIADA 1

/*
 The Private Enterprise Number 32760 has been assigned to Koliada.

 Prefix: iso.org.dod.internet.private.enterprise (1.3.6.1.4.1)

 The IANA PEN registry is available at the following address:

 http://ftp.iana.org/assignments/enterprise-numbers

 Koliada also defines a root ITU-T OID for the identification of all koliada system objects.

 This OID is used to uniquely identify every object:
 	34e25bc0-791b-11dc-b76a-0002a5d5c51b
	http://www.oid-info.com/get/2.25.70295175819406043013049885836617041179

 Third parties may define child objects for the definition of globally unique, non
 overlaping object identifiers.
 */
#define KOLIADA_PEN 32760
#define KOLIADA_OID "34e25bc0-791b-11dc-b76a-0002a5d5c51b"

// Koliada build configuration
// may be done externally or by include
#ifndef __KOLIADA_CONFIG_H
	#ifdef KOLIADA_CONFIG_H
		#include KOLIADA_CONFIG_H
	#else
		#include "KoliadaConfig.h"
	#endif
#endif

// make sure the config set is fully defined
#ifndef USE_SHARED_COMPONENTS
	#define USE_SHARED_COMPONENTS	0
#endif
#ifndef USE_KOLIADA_RTOS
	#define USE_KOLIADA_RTOS 0
#endif
#ifndef USE_THREADS
	#define USE_THREADS 0
#endif
#ifndef USE_MPX_EXTENDED
	#define USE_MPX_EXTENDED 0
#endif
#ifndef USE_TASK_MANAGER
	#define USE_TASK_MANAGER 0
#endif
#ifndef USE_CLOCK_MANAGER
	#define USE_CLOCK_MANAGER 0
#endif

// Koliada toolchain configuration
// done externally and by include
#include "KoliadaConfig.h"
#include "compiler.h"

#ifndef __COMPILER_H
	#error no compiler header defined!
#endif

#include "typedefs.h"		// system typedefs
#include "system.h"			// system API declarations

#include "assert.h"
#include "debug.h"
#include "except.h"
#include "strfmt.h"

#include "sdk.h"
#include "bios.h"

#if 0
#if USE_KOLIADA_EVENT
//	#include "event.h"		// Koliada EVENT support
#endif

#if USE_KOLIADA_AST
//	#include "ast.h"			// Koliada AST support
#elif USE_KOLIADA_EVENT
	#warning USE_KOLIADA_EVENT requires USE_KOLIADA_AST enabled
	#define USE_KOLIADA_AST 1
//	#include "ast.h"
#endif

#if USE_KOLIADA_RTOS
	#include "rtos.h"			// koliada RTOS support
#endif
#endif

// some things must be defined
#ifndef __sizeof_word
	#error __sizeof_word is undefined
#endif

// if you use these, you need to define them somewhere
// typically 'version.c' which includes 'build.h', generated by buildVersion
// your milage may vary!!
extern char *_versionStr;
extern dword _buildNumber;

#endif //__KOLIADA_H
