/*
 *	File:	Koliada - api.h
 *
 *	Contains: KoliadaES API header
 *
 * Edition: TINY
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __API_H
#define __API_H
#include "dataseg.h"
#include "segptr.h"

//#define USE_API_DESC 1
//#define DEBUG_API 1

typedef OBJECT (*API)();
typedef CONST struct ApiEntry
	{
	struct                  // method qualifiers
		{
		word isstatic  :1;	//
		word isvirtual :1;	//
		word inherits  :1;	//
		};
	API fn;
#if USE_API_DESC
	char *desc;
#endif
	} ApiEntry;

typedef CONST struct AlertEntry
	{
	alertType bits;
	const char *fmt;
	} AlertEntry;

// mostly, debuging is controlled via object system API
// occasionally it is handy to have more - particularly during bring up
#if defined(DEBUG) & defined(DEBUG_API)
	#define API_DEBUG(...)	debug(__VA_ARGS__)
#else
	#define API_DEBUG(...)
#endif

#define API_DECL(type, api, args)                                             \
	OBJECT __##type##_##api args

#define CREATE_ENTRY(type, api, args)                                         \
API_DECL(type, api, args) {                                                   \
	OBJECT apiStatus = kNoErr;                                                 \
	WAIT_DS/**/                                                                \
	type *this = SEGPTR(type, obj);/* inside DS */                             \
	API_DEBUG(">>%s.%s(%p, ...)\n", SEGPTR(object, obj)->name, #api, obj);     \
	/*debug("%s: %s.%u\n", __FUNC__, __FILE__, __LINE__);/**/

#define API_ENTRY(type, api, args)                                            \
API_DECL(type, api, args) {                                                   \
	OBJECT apiStatus = kNoErr;                                                 \
	WAIT_DS/**/                                                                \
	type *this = SEGPTR(type, obj);/* inside DS */                             \
	API_DEBUG(">>%s.%s(%p, ...)\n", SEGPTR(object, obj)->name, #api, obj);     \
	/*debug("%s: %s.%u\n", __FUNC__, __FILE__, __LINE__);/**/

#define API_STATIC(type, api, args)                                           \
API_DECL(type, api, args) {                                                   \
	OBJECT apiStatus = kNoErr;                                                 \
	WAIT_DS/**/                                                                \
	/*API_DEBUG(">>%s.%s(%p, ...)\n", SEGPTR(object, obj)->name, #api, obj);/**/     \
	/*debug("%s: %s.%u\n", __FUNC__, __FILE__, __LINE__);/**/

#define API_EXIT(type, api)                                                   \
apiDone:;                                                                     \
	/*debug("%s: %s.%u\n", __FUNC__, __FILE__, __LINE__);/**/                  \
	/*API_DEBUG("<<%s.%s(%x)\n", SEGPTR(object, obj)->name, #api, apiStatus);/**/    \
	POST_DS/**/                                                                \
	return apiStatus;};

#define ApiError(obj, fn, ...)\
	do{apiStatus=__object_alert(__FILE__, __LINE__, obj, fn, __VA_ARGS__);goto apiDone;}while(0)
#define ApiReturn(arg)				do{apiStatus=(OBJECT)arg;goto apiDone;}while(0)
#define ApiDone()						do{goto apiDone;}while(0)

#define BeginClassDispatchTable(name)\
STATIC CONST ApiEntry __##name##DispatchTable[]={\
	// entries should be preserved, in order, from version to version
	// depreciated function numbers should not be removed (backwards compatibility)
	// shared libs should be able to 'inherit' from a parent (add ->parent)

#if USE_API_DESC
	#define apiEntry(class, name, args)	(API) &__##class##_##name, #name "(" args ")"
#else
	#define apiEntry(class, name, args)	(API) &__##class##_##name
#endif
#define apiUndef(class, name, args)	(API) NULL

#if USE_API_DESC
	#define EndClassDispatchTable(name)\
		{(API) 0, (char *)0}};/* must end with a null entry! */
#else
	#define EndClassDispatchTable(name)\
		{0, 0, 0, (API) -1}};/* must end with a special entry! */
#endif

#define ExternApi(class, name) 	extern OBJECT __##class##_##name()

#define BeginClassAlertTable(name)\
STATIC CONST AlertEntry __##name##AlertTable[] = {\
	/* 0x00 */ {kNoErr, ""},	/* reserved - must be defined */\
	// entries should be preserved, in order, from version to version
	// depreciated error numbers should not be removed (backwards compatibility)
	// error bits set by a given error may/can change

#define EndClassAlertTable(name)\
	{kNoErr, (char *)0}};/* must end with a null entry! */

#if !RELEASE
#ifndef __MSC
	#warning uses __object_alert
#endif
#endif
OBJECT __object_alert(char *file, word line, OBJECT obj, ObjectName method, ObjectClass type, word errno, ...);

#endif//__API_H

