/*
 *	File: Koliada - debug.h
 *
 * Contains: Koliada Debug interface
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __DEBUG_H
#define __DEBUG_H
// DEBUG - debug code is ON by default
// NODEBUG - debug code is included but OFF by default
// RELEASE - debug code is excluded
#if M4_MACROS
// m4 is used to exclude code in RELEASE versions
define(BeginDebug,
	#if !RELEASE
	if (__debugArg & 0x01) {
	#line $1
	)
define(EndDebug,
	}
	#endif
	#line $1
	)
#endif

// just in case M4 is not run (to exclude code in RELEASE versions)
#define BeginDebug(__line__) if(__debugArg & 0x01){
#define EndDebug(__line__)	}

extern unsigned __debugArg;

#if 0//_WIN32
	#if _DEBUG
		#define DEBUG 1
	#endif
	//   #define printf debug
	#if debug
	   extern int debug(const char *, ...);
	#else
		extern int debug(const char *, ...);
	#endif
		char *__file(char *);
		void __fsWaitDebug(void);
		void __fsPostDebug(void);
#elif __GNUC__
#include <string.h>
	#define __fsWaitDebug()
	#define __fsPostDebug()
//	#define __file(f) &strrchr(f, '/')[1]
static inline char *__file(char *f)
	{
	char *s = strrchr(f, '/');
	return s ? &s[1] : f;
	}
#else
	#define __fsWaitDebug()
	#define __fsPostDebug()
	#define __file(f) f
#endif

//	__do(char *spinChar="|/-\\";static unsigned n;debug("%c\b", spinChar[n%4]);n++;)
#define _spinchar() __do(char *spinChar="|/-\\";static unsigned n;debug("%c\b", spinChar[n%4]);n++;)

#if DEBUG
	#define SHOW_DEBUG(bit)	(__debugArg |=  kDebug)
	#define HIDE_DEBUG(bit)	(__debugArg &= ~kDebug)

	#if 1
	   #define DEBUG_PRINTF(args)		do if (__debugArg & kDebug) { debug("%s.%d: ", __file(__FILE__), __LINE__); debug args; } while(0)
		#define DEBUG_MORE(args)	   do if (__debugArg & kDebug) { debug args; } while(0)
//		#define DEBUG_LIB(lib, bits)	do if (__debugArg & kDebug) { libSetDebugMask(lib, bits); } while(0)
	#else
	   #define DEBUG_PRINTF(args)	do { debug("%s.%d: ", __file(__FILE__), __LINE__); debug args; } while(0)
		#define DEBUG_MORE(args)   do { debug args; } while(0)
	#endif

	int __newline(char *fmt, ...);
	void __line(int (*)(const char *, ...), char *, int);
	#define _trace __line(debug, __FILE__, __LINE__)
	#define Trace(x) do{__line(debug, __FILE__, __LINE__); debug x; __newline x;}while(0)
#else
	#define SHOW_DEBUG(bit)
	#define HIDE_DEBUG(bit)
   #define DEBUG_PRINTF(args)
	#define DEBUG_MORE(args)
	#define DEBUG_LIB(lib, bits)

	#define _trace
	#define Trace(x)
#endif

#define ShowDebug(bit)	SHOW_DEBUG(bit)
#define HideDebug(bit)	HIDE_DEBUG(bit)

#define __STR(z) __TMP(z)
#define __TMP(z) #z

#if 0
#define test(t) ((void) ((t) ? ((void) 0) : \
				(debug("Assertion failed: " #t " in " __FILE__ ", line " __STR(__LINE__) "\n"), exit(0))))
#endif

#if 0 // old
	#if DEBUG
		#define DEBUG_PRINTF(args) do{debug("%s.%d: ", __FILE__, __LINE__); debug args;}while(0)
		#define DEBUG_MORE(args)   debug args;
		#define DEBUG_CHECK(var,name)\
			do{static void * local##var = (void *) var;\
			if (local##var != var) {debug("%p,%p - "name" trashed!!\n", var, local##var); __fsCall(0);}}while(0)
	#else
		#define DEBUG_PRINTF(args)
		#define DEBUG_MORE(args)
		#define DEBUG_CHECK(var,name)
	#endif
#endif // old

#if __GNUC__
	int debug(const char *, ...) __attribute__ ((format (printf, 1, 2)));
#else
	int debug(const char *, ...);
#endif

#endif
