/*
 * File: Koliada pCache - pCache.h
 *
 * Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __PCACHE_H
#define __PCACHE_H
#include "Koliada.h"

#define PCACHE_VERSION	0x0107	// current pCache version

#if CC8051
	/*
	 * The following rules apply when writing multiple times to a 32-bit word between erase:
	 * 	 Writing 0 to a bit within a 32-bit flash word, which has been set to 1 by the last erase operation,
	 * 	  changes the state of the bit to 0, subject to the last bullet below.
	 * 	 It is possible to write 0 to a bit within a 32-bit word repeatedly (subject to the last bullet below) once the
	 * 	  bit has been written with 0. This does not change the state of the bit.
	 * 	 Writing 1 to a bit does not change the state of the bit, subject to the last bullet below.
	 * 	 The following limitations apply to writes subsequent to the last page erase:
	 * 		 A 0 must not be written more than two times to a single bit.
	 * 		 A 32-bit word shall not be written more than 8 times.
	 * 		 A page must not be written more than 1024 times.
	 * The state of any bit of a 32-bit flash word is nondeterministic if these limitations are violated.
	 *
	 * This makes it possible to write up to 4 new bits to a 32-bit word up to 8 times.
	 */
#endif
#if 0
	#define HEAD_CHUNK		0xf0f0	// first chunk
	#define CONT_CHUNK		0xf8f8	// continuation chunk
	#define RUNT_CHUNK		0xfcfc	// tail chunk about to become a continuation chunk
	#define TAIL_CHUNK		0xfefe	// tail chunk

	// CMASK values prevent writing multiple zeros
	#define HEAD_CMASK		0xf7f7	// 0111
	#define CONT_CMASK		0xfbfb	// 1011
	#define RUNT_CMASK		0xfdfd	// 1101
	#define TAIL_CMASK		0xfefe	// 1110
#else
	#define HEAD_CHUNK		0xfff0	// first chunk
	#define CONT_CHUNK		0xfff8	// continuation chunk
	#define RUNT_CHUNK		0xfffc	// tail chunk about to become a continuation chunk
	#define TAIL_CHUNK		0xfffe	// tail chunk

	// CMASK values prevent writing multiple zeros
	#define HEAD_CMASK		0xfff7
	#define CONT_CMASK		0xfffb
	#define RUNT_CMASK		0xfffd
	#define TAIL_CMASK		0xfffe
#endif


#define kFirstpCacheType	0x10	// first 15 types are reserved for pCache

typedef unsigned SIZE_T;
typedef void *ROMPTR;

typedef struct chunk
	{
	wyde magic;			// magic marker
	wyde len;			// size of the data
	wyde crc;			// crc16 of the chunk data
	wyde type;			// callers (non zero) object type
	byte data[0];
	} chunk;

// scan return values
#define kScanContinue 0
#define kScanStop 1
#define kScanDeleteContinue 2
#define kScanDeleteStop 3

STATUS pCacheInit(ROMPTR, SIZE_T count, SIZE_T bsize, SIZE_T rsize, boolean (*flushFn)(wyde type, void *buf, SIZE_T size), bool reset);
STATUS newChunk(wyde type, void *buf, SIZE_T size);
void *locChunk(wyde type, void *key, int (*compare)(void *, void *, SIZE_T));
STATUS delChunk(wyde type, void *key, int (*compare)(void *, void *, SIZE_T));
STATUS scanCache(wyde type, void *key, void *cookie, STATUS (*handler)(void *, void *, void *, SIZE_T));
void pCacheAnalyse();
void eraseCache(void *addr, SIZE_T count, SIZE_T size);

#endif//__PCACHE_H
