/*
 * File: Koliada - strfmt.h
 *
 * Contains: Koliada raw C string formatter
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __STRFMT_H
#define __STRFMT_H
#include <stdarg.h>
#include "ctypes.h"

// outputs formatted chars to a string or a function

typedef union
	{
	char *buf;
#if USE_ALT_OUTCH_T
#warning !!!!!!!!!!!!!!!!!!
	int (*out)(word arg);
#else
	int (*out)(int c);
#endif
	} outch_t;

// strfmt uses n (bufsize) to disambiguate string and function handlers
// if n == 0 outc is assumed to hold a function pointer
// be sure to make sure any string handlers don't call using n = 0!

#if USE_STRFMT_BIOS
#include "target.h"
	// map STRFMT to the bios strfmt function
	// this is generally preferable, since _strfmt is typically relatively large
#ifndef _sys
	#define _sys (&sysData)
#endif
	#define STRFMT ((int (*)(outch_t, unsigned, const char *, va_list)) _sys->strfmt)
#endif

#ifndef STRFMT
	// use statically linked strfmt
	int _strfmt(outch_t outch, unsigned n, const char *fmt, va_list args);
	#define STRFMT _strfmt
#endif

// binary ("%b"), real number ("%[e,f,g]"), and IP address ("%i") are all deployment specific

// strbuf and strnbuf call _strfmt to format into a string buffer
size_t strbuf(char *dst, size_t i, const char *fmt, ...);
size_t strnbuf(const size_t size, char *dst, size_t i, const char *fmt, ...);

#endif//__STRFMT_H

