/*
 *	File: Koliada - typedefs.h
 *
 *	Contains: Koliada system typedefs
 *
 *		Generally included by the specific target header
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef	__TYPEDEFS_H
#define	__TYPEDEFS_H
#include "ctypes.h"
// pulled in by Koliada.h

// alert bits
#if 1
typedef enum
	{
	// fits in 4 bits
	kNoErr	= 0x0,
	kInfo		= 0x1,			// info
	kDebug	= 0x2,			// debug
	kError	= 0x4,			// error (function failure)
	kFatal	= 0x8,			// fatal (system failure)
	} alertType;
#else
	#define 	kNoErr	0x0
	#define 	kInfo		0x1			// info
	#define 	kDebug	0x2			// debug
	#define 	kError	0x4			// error (function failure)
	#define 	kFatal	0x8			// fatal (system failure)
#endif
#define kNoError kNoErr

#if 0
typedef enum
	{
	// fits in 4 bits
	kNone		= 0x0,			// no action
	kRaise	= 0x1,			// raise - raises an exception (unsupported)
	kDebug	= 0x8,			// debug - do something to debug
	} alertAction;
#endif

// Koliada object identifiers
typedef OBJECT CPU;
typedef OBJECT NODE;
typedef OBJECT TASK;
typedef OBJECT PIPE;
typedef OBJECT CLASS;
typedef OBJECT EVENT;
typedef OBJECT MUTEX;
typedef OBJECT QUEUE;
typedef OBJECT TIMER;
typedef OBJECT ACTION;
typedef OBJECT SIGNAL;
#ifndef __MSC
typedef OBJECT HANDLE;
#endif
typedef OBJECT LIBRARY;		// synonymous with COMPONENT
typedef OBJECT MAILBOX;
typedef OBJECT CALLBACK;
typedef OBJECT RESOURCE;
typedef OBJECT SEMAPHORE;
typedef OBJECT COMPONENT;	// synonymous with LIBRARY

//typedef void (*AST_FN)();
typedef void (*THREAD)();
typedef void (*HANDLER)();
typedef void (*InterruptHandler)();

typedef OBJECT (*METHOD)();// object method ptr

#if 0
// Koliada object types
typedef enum
	{
	/* 0 */	kNulType,
	/* 1 */	kCpuType,
	/* 2 */	kTcbType,
	/* 3 */	kMuxType,
	/* 4 */	kSemType,
	/* 5 */	kAstType,
	/* 6 */	kTaskType,
	/* 7 */	kPipeType,
	/* 8 */	kEventType,
	/* 9 */	kClassType,
	/* a */	kQueueType,
	/* b */	kTimerType,
	/* c */	kSignalType,
	/* d */	kLibraryType,
	/* e */	kMailboxType,
	/* f */	kResourceType,
	} ObjectType;				// not more than 63 types
typedef word ObjectAttrib;
#define kMaxObjectTypes 16
typedef TASK TaskId;
#endif

#if !RELEASE
extern char CONST *kObjectTypeNames[];
#endif

#if USE_SHARED_COMPONENTS

// shared library defs
typedef STATUS (*SHARED)();
typedef struct SharedEntry
	{
	SHARED fn;
	char *desc;
	} SharedEntry;
typedef word SharedDispatchTable;

typedef struct SharedAlert
	{
	alertType bits;
	char *fmt;
	} SharedAlert;
typedef SharedAlert SharedAlertTable[];

// lib alert level action masks
typedef enum
	{
	// fit in 4 bits!
	// overlays alertType above!
	kAlertInfo   = 0x1,	// info on/off
	kAlertDebug  = 0x2,	// debug on/off
	kAlertError	 = 0x4,	// error on/off
	kAlertCalls	 = 0x8,	// lib->call entry/exit on/off
	kAlertAll	 = 0xf,	// all of the above
	} SharedAlertMask;

typedef enum
	{
	kQueueFIFO = 0x01,
	kQueueLIFO,
	kQueueOrdered,
	kQueuePriority,
	} QueueType;				// not more than 14 types

typedef enum
	{
	kIterateLast = 0x01,
	kIterateRandom,
	kIterateInorder,
	kIteratePreorder,
	kIteratePostorder,
	} IterateOp;

typedef STATUS (*IterateHandler)(QUEUE, OBJECT, OBJECT);

#define kMailMsgSize 4
typedef struct _MAILMSG
	{
	UInt32 content[kMailMsgSize];
	} *MAILMSG_T;
typedef void *MAILMSG;

typedef enum
	{
	kWaitNot = 0x01,
	kWaitPoll,
	kWaitSuspend,
	kWaitFifo,
	kWaitPriority,
	kMsgPostBusy,
	kMsgWaitNone,
	} WaitMode;

typedef char *DeviceId;
typedef char *DeviceDesc;

#endif// USE_SHARED_COMPONENTS

#ifndef min
	#define min(a, b) ((a) < (b) ? (a) : (b))
#endif
#ifndef max
	#define max(a, b) ((a) > (b) ? (a) : (b))
#endif
#ifndef LAMBDA
	#define LAMBDA ((void *)-1)
#endif
#ifndef NULL
	#define NULL ((void *)0)
#endif

#define __FUNC__ __func__

#endif	//__TYPEDEFS_H
