/*
 *	File: Koliada - BoardConfig.h
 *
 *	Contains: Architecture specific BoardConfig 'base' class
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2015 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __BOARDCONFIG_H
#define __BOARDCONFIG_H
#include "target.h"

#include "gpio.h"
#include "uart.h"

// BoardConfig 'base class'

// board configuration data allows the system to determine configuration at runtime
// data is stored in flash and referenced by trusted modules via a well known location

// Koliada abstracts the access to gpios, UARTs and LEDs via mapping tables in the
// BoardConfig data object. This allows for binary component compatability across
// systems.

// Any given system is not required to have any or all of these items.

// BoardConfig structure version
//#define BD_STRUCT_VERSION ((UInt32) 0x2dc50beb)// Tue Aug 02 16:47:43 2011
	// first cut for CC25XX
#define BD_STRUCT_VERSION ((UInt32) 0x36733025)// Fri Oct 25 19:00:37 2013
	// extended to provide a memory map

//#define USE_ST_TICK	1	// sleep timer
//#define USE_T3_TICK	1	// Timer 3
#define USE_T2_TICK	1	// Timer 2
#define USE_CMTIMER  1	// event timers

#define __BOARDCONFIG_BASE_CLASS\
	BOARD_CONFIG_HEADER        /* constant across all architectures                 */\
	\
	word clocksPerSecond;		/* system clock rate (Hz)                            */\
	word systemTickRate;			/* system tick rate (Hz)                             */\
	byte clocksPerTick;			/* system clock to tick divisor                      */\
	\
	PortConfig ports;				/* gpio config	                                      */\
	PinConfig  pins;				/* pin config                                        */\
	\
	UartConfig uarts;				/* uart config                                       */\
	word port;						/* debug port                                        */\
	UInt32 baud;					/* debug baud                                        */\
	\
	/* system params                                                                */\
	char	 versionStr[64];		/* version of this board config                      */\
	UInt32 sysVersion;			/* version of sdk system was built under             */\
	UInt32 sysBuildStamp;		/* build stamp of SDK system was built under         */\
	UInt32 sysOldestStamp;		/* oldest module that will work with this system     */\
	char   sysVersionStr[64];	/* system version string                             */\
   \
	PTR bootRomAddr[2];			/* 2x boot rom locations                             */\
	\
	const char **argvLoc[2];   /* boot rom start args                               */\
	\
	const char *const envp;	   /* boot rom envp                                     */\

#ifndef __BOARD_H
typedef const struct BoardConfig
	{
	__BOARDCONFIG_BASE_CLASS
	} BoardConfig;
#endif
#define BD_MAGIC ((UInt16) 'bd')

#define __BoardCFG ((BoardConfig *)_sys->board)

enum	// system config bits
	{
	// bits 0-7 are reserved
	kUse32kHzXtal = BIT0,
	//
	// bits 8-16 may be used by application
	};

#define SYSTEM_TICK_RATE	((BoardConfig *)_sys->board)->systemTickRate
#define CLOCKS_PER_SECOND	((BoardConfig *)_sys->board)->clocksPerSecond

#endif//__BOARDCONFIG_H

