/*
 *	File: BoardConfigCC8051.c
 *
 *	Contains: CC8051 configuration details
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
////////////////////////////////////////////////////////////////////////////////
//
// DANGER - There be dragons below!
//
////////////////////////////////////////////////////////////////////////////////
#define PIN_TABLE_SIZE (sizeof(pinTable)/sizeof(PinDescriptor))// in users config

// Bauderate mantissa and exponent settings
// Baudrate = (((256 + BAUD_M) * 2^BAUD_E) * F) / 2^28
// Baudrate =  115200 bps (BAUD_M = 216, BAUD_E = 11)
// Baudrate =  230400 bps (BAUD_M = 216, BAUD_E = 12)
// Baudrate =  230400 bps (BAUD_M = 216, BAUD_E = 13)
// Baudrate =  921600 bps (BAUD_M = 216, BAUD_E = 14)
// Baudrate = 1843200 bps (BAUD_M = 216, BAUD_E = 15)
// Baudrate = 1996094 bps (BAUD_M = 255, BAUD_E = 15)
// Baudrate = 3992188 bps (BAUD_M = 255, BAUD_E = 16)
#if BAUD_RATE == 115200
	#define UART_BAUD_M  216
	#define UART_BAUD_E  11
#endif
#if BAUD_RATE == 921600
	#define UART_BAUD_M  216
	#define UART_BAUD_E  14
#endif
#if BAUD_RATE == 1996094
	#define UART_BAUD_M  255
	#define UART_BAUD_E  15
#endif
#if !defined(BAUD_RATE) || !defined(UART_BAUD_M)
	#error Undefined BAUD_RATE!
#endif

// BoardData segment
#pragma constseg=BOARDDATA // board data segment (don't mess with this)!

/// Uart Descriptor Table
// describes a uart to the system
const UartDescriptor uartTable[] =
	{
	{ 0x86, 0xC4, 0xC5, 0xC1, 0xC2, {0x0, UART_BAUD_E}, UART_BAUD_M },// UART 0
	{ 0xF8, 0xFB, 0xFC, 0xF9, 0xFA, {0x1, UART_BAUD_E}, UART_BAUD_M },// UART 1
	};
#define UART_TABLE_SIZE (sizeof(uartTable)/sizeof(UartDescriptor))

// Port Descriptor Table
// describes an i/o port to the system
const PortDescriptor portTable[] =
	{
	{ 0x7080, 0x70FD, 0x70F3, 0x708F, 0x7089, 0x70AB, 0x0d},	// P0
	{ 0x7090, 0x70FE, 0x70F4, 0x70F6, 0x708A, 0x708D, 0x0f},	// P1
	{ 0x70A0, 0x70FF, 0x70F5, 0x70F7, 0x708b, 0x70AC, 0x06},	// P2
	};
#define PORT_TABLE_SIZE (sizeof(portTable)/sizeof(PortDescriptor))

#if 0
extern const char envp[];
extern const char *const argv0[];
extern const char *const argv1[];
#endif

// these are SDK specific...
#define BOOT_ADDR_0 ((PTR)0x00006010)	// primary boot entrypoint
#define BOOT_ADDR_1 ((PTR)0x00008000)	// secondary boot entrypoint

// BoardConfig segment
#if __ICC8051__
	#pragma constseg=BOARDCONFIG
	// board config data segment

	// BoardData is a required item
	BoardConfig BoardData;// forward decl
	#pragma required=BoardData
#endif
BoardConfig BoardData =
	{
	BOARD_NAME,
	BD_MAGIC,
	BD_STRUCT_VERSION,
	0xffff, // checksum (if used)
	BOARD_MAJOR,
	BOARD_MINOR,
	BOARD_CONFIG,
	SYSTEM_CONFIG,

	// system clock details (don't mess with this!)
#if USE_ST_TICK
	32768, 2048, 16,	// using the sleep timer as the system clock
#endif
#if USE_T3_TICK
	2000, 2000, 1,	// using the T3 as the system clock
#endif
#if USE_T2_TICK
//	1000, 1000, 0x7C,	// using the T2 as the system clock
	2000, 2000, 0x3E,	// using the T2 as the system clock
#endif

	// gpio port descriptors
	{ PORT_TABLE_SIZE, (PortDescriptor *) portTable },

	// gpio pin descriptors
	{ PIN_TABLE_SIZE, (PinDescriptor *) pinTable },

	// uart descriptors
	{ UART_TABLE_SIZE, (UartDescriptor *) uartTable },

	// debug uart & speed
	DEBUG_UART, BAUD_RATE,

	__BUILD_VERSION_STR,
	kSdkVersion, kSdkBuildStamp, kSdkOldestStamp, __KOLIADA_SDK,

	// BootROM addresses
	{BOOT_ADDR_0, BOOT_ADDR_1},

	// BootROM args
	(PTR) &argv0, (PTR) &argv1,

	// BootROM envp
	(PTR) &envp,
	};

////////////////////////////////////////////////////////////////////////////////
#if __ICC8051__
	#pragma constseg=default
#endif
////////////////////////////////////////////////////////////////////////////////

#if CONFIG_LOADER
// this is thrown away post load

#if 0
static UInt16 _sysBoardDataChecksum(const BoardConfig *bd)
	{
	UInt16 cksum = 0;
	UInt16 *p;
	int i, len = sizeof(BoardConfig) >> 2;

	p = (UInt16 *)bd;
	for (i = 0; i < len; i++)
		cksum += p[i];

	return cksum;
	}
#endif

#define BoardConfig __main
void showBoardConfig();
void showGpio();
int BoardConfig(int argc, char **argv);// user config exec

int main(int argc, char **argv)
	{
	debug("%s\n", BoardData.versionStr);

#if 0
	// calculate the checksum
	UInt16 csum = _sysBoardDataChecksum(&BoardData);
	...
#endif

	showBoardConfig();
	showGpio();

	debug("Loaded\n");
	return BoardConfig(argc, argv);// non zero return will start otaLoader (if installed)
	}
#endif
