/*
 * File: Koliada - KoliadaOptions.h
 *
 * Contains: Koliada Configuration Header
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2011 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifdef __KOLIADA_H// special usage - only pull in thru Koliada.h
// make a local copy and configure for your target
// do not include directly - file is included via Koliada.h

// TBD - create this file automagically from user config

#define USE___TINY_MODEL            1     // TINY Embedded System
#define USE_POOLED_MODEL            1     // POOLED Embedded System
#define USE_SHARED_MODEL            1     // SHARED Embedded System

#if USE___TINY_MODEL
	// check for the other models!

	#define USE_MODULE_DATA          0     // set if using module data
	#define USE_SHARED_COMPONENTS    0     // set if using shared components
	#define USE_KOLIADA_RTOS         0

	// TINY ES
	#define USE_KOLIADA_AST          1
	#define USE_KOLIADA_EVENT        1
	#define USE_KOLIADA_TIMER        1
	#if USE_KOLIADA_AST
		#define kActionSize          13     // words
		#define kActionTableSize     16     // ASTs
	#endif
	#if USE_KOLIADA_EVENT
		#define kMaxEventArgs              4   // words
		#define kEventTableSize            16  // EVENTs
		#define kEventSleepNotifyTableSize 4   // EVENTs
		#define kEventAwakeNotifyTableSize 2   // EVENTs
	#endif
	#if USE_KOLIADA_TIMER
		#define kTimerSize           11     // words
		#define kTimerTableSize      16     // events
	#endif
#endif

#if USE_KOLIADA_RTOS
	#define USE_TASK_MANAGER			0     //
	#define USE_THREADS              0     // must be 1 for TM, default is 1 otherwise
	#define USE_STATIC_MEMPOOLS		1     // must be 1 for non-threaded, default 0 for threaded & TM

	// can set object name sizes individually and/or collectively
	// sem & mux name sizes will always be the same (mux namesize prevails)
#if 1 // for example:
	#define kDefaultNameSize			8		// default is 16
#else // or
	#define kMuxNameSize					kDefaultNameSize
	#define kSemNameSize					kDefaultNameSize
	#define kTaskNameSize				kDefaultNameSize
#endif

	// memory pools can be static or dynamic
	#if USE_STATIC_MEMPOOLS
		// can set object pool sizes individually and/or collectively
	#if 0 // for example:
		#define kDefaultTableSize		8		// default is 16
	#else // or
		#define kMaxTcbTableSize     12     // default is kDefaultTableSize
		#define kMaxMuxTableSize		8     // default is kDefaultTableSize
		#define kMaxSemTableSize		4     // default is kDefaultTableSize
	#endif
	#endif
#endif

#if 0
#if !RELEASE
	#warning fixme - USE_CMTIMER is duplicated!
#endif
	#define USE_CMTIMER                 0     // include the cm library
	#if USE_CMTIMER
		#define USE_CMIDLE					0     // CM idle task
		#define kCmTickRate	         1000     // default is 1000
		#define kMaxTimerTableSize      12     // default is 16
		#if USE_THREADS
		#if 0
			#define kCmIdleStackSize            // default is kMinStackSize
			#define kCmTimerStackSize           // default is kMinStackSize
		#endif
		#endif
	#else
	#endif
#endif

#define USE_CMDLINE                 0      // include the cmdline bios extensions

#else //__KOLIADA_H
#warning Invalid use of "KoliadaConfig.h"
#endif//__KOLIADA_H

