#ifndef __BITDEFS_H
#define __BITDEFS_H

/*******************************************************************************
 * Interrupt Control Registers
 */

// IEN0 (0xA8) - Interrupt Enable 0 Register - bit accessible SFR register

// IEN1 (0xB8) - Interrupt Enable 1 Register - bit accessible SFR register

// IEN2 (0x9A) - Interrupt Enable 2 Register
#define IEN2_WDTIE                        0x20
#define IEN2_P1IE                         0x10
#define IEN2_UTX1IE                       0x08
#define IEN2_I2STXIE                      0x08
#define IEN2_UTX0IE                       0x04
#define IEN2_P2IE                         0x02
#define IEN2_USBIE                        0x02
#define IEN2_RFIE                         0x01


// TCON (0x88) - CPU Interrupt Flag 1 - bit accessible SFR register

// S0CON (0x98) - CPU Interrupt Flag 2 - bit accessible SFR register

// S1CON (0x9B) - CPU Interrupt Flag 3
#define S1CON_RFIF_1                      0x02
#define S1CON_RFIF_0                      0x01


// IRCON (0xC0) - CPU Interrupt Flag 4 - bit accessible SFR register

// IRCON2 (0xE8) - CPU Interrupt Flag 5 - bit accessible SFR register

// IP1 (0xB9) - Interrupt Priority 1
#define IP1_IPG5                          0x20
#define IP1_IPG4                          0x10
#define IP1_IPG3                          0x08
#define IP1_IPG2                          0x04
#define IP1_IPG1                          0x02
#define IP1_IPG0                          0x01

// IP0 (0xA9) - Interrupt Priority 0
#define IP0_IPG5                          0x20
#define IP0_IPG4                          0x10
#define IP0_IPG3                          0x08
#define IP0_IPG2                          0x04
#define IP0_IPG1                          0x02
#define IP0_IPG0                          0x01



#define PICTL_PADSC                       0x40
#define PICTL_P2IEN                       0x20
#define PICTL_P0IENH                      0x10
#define PICTL_P0IENL                      0x08
#define PICTL_P2ICON                      0x04
#define PICTL_P1ICON                      0x02
#define PICTL_P0ICON                      0x01


// T3CNT (0xCA) - Timer 3 Counter

// T3CTL (0xCB) - Timer 3 Control
#define TxCTL_DIV                         0xE0
#define TxCTL_DIV0                        0x20
#define TxCTL_DIV1                        0x40
#define TxCTL_DIV2                        0x80
#define TxCTL_START                       0x10
#define TxCTL_OVFIM                       0x08
#define TxCTL_CLR                         0x04
#define TxCTL_MODE                        0x03
#define TxCTL_MODE0                       0x01
#define TxCTL_MODE1                       0x02

#define TxCTL_DIV_1                       (0x00 << 5)
#define TxCTL_DIV_2                       (0x01 << 5)
#define TxCTL_DIV_4                       (0x02 << 5)
#define TxCTL_DIV_8                       (0x03 << 5)
#define TxCTL_DIV_16                      (0x04 << 5)
#define TxCTL_DIV_32                      (0x05 << 5)
#define TxCTL_DIV_64                      (0x06 << 5)
#define TxCTL_DIV_128                     (0x07 << 5)

#define TxCTL_MODE_FREERUN                (0x00)
#define TxCTL_MODE_DOWN                   (0x01)
#define TxCTL_MODE_MODULO                 (0x02)
#define TxCTL_MODE_UPDOWN                 (0x03)

// TxCCTL0 (0xCC) - Timer 3/4 Channel 0 Compare Control
#define TxCCTL0_IM                        0x40
#define TxCCTL0_CMP                       0x38
#define TxCCTL0_CMP0                      0x08
#define TxCCTL0_CMP1                      0x10
#define TxCCTL0_CMP2                      0x20
#define TxCCTL0_MODE                      0x04

#define TxC0_SET_ON_CMP                   (0x00 << 3)  // Set output on compare
#define TxC0_CLR_ON_CMP                   (0x01 << 3)  // Clear output on compare
#define TxC0_TOG_ON_CMP                   (0x02 << 3)  // Toggle output on compare
#define TxC0_SET_CMP_UP_CLR_0             (0x03 << 3)  // Set output on compare-up clear on 0
#define TxC0_CLR_CMP_UP_SET_0             (0x04 << 3)  // Clear output on compare-up set on 0
#define TxC0_SET_CMP_CLR_255              (0x05 << 3)  // Set when equal to T3CC0, clear on 255
#define TxC0_CLR_CMP_SET_0                (0x06 << 3)  // Clear when equal to T3CC0, set on 0

// T3CC0 (0xCD) - Timer 3 Channel 0 Compare Value

// T3CCTL1 (0xCE) - Timer 3 Channel 1 Compare Control

#define TxCCTL1_IM                        0x40
#define TxCCTL1_MODE                      0x04
#define TxCCTL1_CMP                       0x38
#define TxCCTL1_CMP0                      0x08
#define TxCCTL1_CMP1                      0x10
#define TxCCTL1_CMP2                      0x20

#define TxC1_SET_ON_CMP                   (0x00 << 3)  // Set output on compare
#define TxC1_CLR_ON_CMP                   (0x01 << 3)  // Clear output on compare
#define TxC1_TOG_ON_CMP                   (0x02 << 3)  // Toggle output on compare
#define TxC1_SET_CMP_UP_CLR_0             (0x03 << 3)  // Set output on compare-up clear on 0
#define TxC1_CLR_CMP_UP_SET_0             (0x04 << 3)  // Clear output on compare-up set on 0
#define TxC1_SET_CMP_CLR_C0               (0x05 << 3)  // Set when equal to T3CC1, clear when equal to T3CC0
#define Tx3C1_CLR_CMP_SET_C0               (0x06 << 3)  // Clear when equal to T3CC1, set when equal to T3CC0

// T3CC1 (0xCF) - Timer 3 Channel 1 Compare Value

#endif//__BITDEFS_H