/*
 *	File: Koliada - flash.h
 *
 *	Contains: basic flash i/o for use during bootstrap/debug
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright:	 1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __FLASH_H
#define __FLASH_H

#define kFlashPageSize 2048

#define FlashAddr(a) ((word) ((a) >> 2))
#define FlashBank(a) ((byte)  (((UInt32) a) / kFlashPageSize)/16)
#define FlashBankAddr(b) ((((UInt32) b)* kFlashPageSize)*16)
#define FlashPage(a) ((byte) (((UInt32) a) / kFlashPageSize))
#define FlashPageOffset(a) ((word) (((UInt32) a) - (((UInt32) a) / kFlashPageSize) * kFlashPageSize))
#define FlashPageAddr(p) ((((UInt32) (p))* kFlashPageSize) >> 2)

// FlashTBL	                              // flash function table
typedef CONST struct FlashTBL
	{
	void (*erase)(byte page);								// erase a flash page
	void (*write)(word loc, byte *buf, word len);	// write ram to flash
	void (*fetch)(word loc, byte *buf, word len);	// fetch flash to ram
	void (*copy)(word dst, word src, word len);		// copy flash to flash
	// these are here for want of a better place to put them...
	char *(*getenv)(const char *name);
	int (*setenv)(const char *name, const char *value, int overwrite);
	} FlashTBL;
// flash transfer len is # 32bit words

// FlashTBL is not extensible, but it is replacable via _sys->fios
// replacements must match this vtable!

// library entry points that redirect via FlashTBL
void sysFlashErase(byte page);

// notes:
//    loc, src, dst are 18 bit abosolute addresses >> 2 bits (4 byte aligned)
//    buf MUST be in physical >RAM< (not just XDATA)
//    count is # UInt32
void sysFlashWrite(word loc, byte *buf, word count);
void sysFlashFetch(word loc, byte *buf, word count);
void sysFlashCopy(word dst, word src, word count);

#endif//__UART_H
