/*
 *	File: radio.h
 *
 *	Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2013 by Guy McIlroy.
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __RADIO_H
#define __RADIO_H

// iocntl options
enum
	{
	// 0x00-0xff are reserved
	kRfSetChannel = 0x0100,	// value in range 11 <> 26
	kRfSetRxBuf,				// byte *buf, byte len);
	kRfSetTxPower,				// value in range 0 <> 15
	kRfSetTestMode,		// places transmitter in specific test mode
	kRfGetRssiOffset,			// get the RSSI offset for the radio
	kRfGetMaxFrameSize,		// get the maximum RF frame size
	kRfSetFrameSig,			// set the signature to use
	kRfGetFrameSig,			// get the signature in use
	};

void initRadio();									// init radio for use (inits to idle)

int rfIocntl(word request, ...);				// set/get enumerated RF data values

bool (*rfGetRxEvent())(byte *buf, byte len);
bool (*rfSetRxEvent(bool (*handler)(byte *buf, byte len), byte *buf, byte len))(byte *buf, byte len);

void (*rfGetTxDone())();						// get the TX done handler
void (*rfSetTxDone(void (*handler)()))();	// set the TX done handler (always chain txDone!!)

void (*rfGetError())();							// get the RF error handler
void (*rfSetError(void (*handler)()))();	// set the RF error handler

bool rfSendFrame(byte *p, word len);		// send a frame buffer

byte rfGetChannel();								// get the current channel
byte rfSetChannel(byte value);				// set the current channel
byte rfGetTxPower();								// get the current TX power level
byte rfSetTxPower(byte value);				// set the current TX power level

void rfEnableRx();								// enable RX
bool rfToggleRx();								// toggle the RX state
void rfRadioOff();								// set radio to lowest power
word rfRadioStatus();							// returns the radio status word (device specific)

void waitRadio();									// stop  radio interrupts
void postRadio();									// start radio interrupts

bool rfTxDone();									// true when frame TX done
bool rfRxDone();									// true when frame RX done
bool rfTxInProgress();							// true while frame is transmitting
bool rfRxInProgress();							// true while frame is incomming

byte *rfGetBuffer();								// get current input buffer
byte  rfGetBufferSize();						// get current input buffer size
byte *rfSetBuffer(byte *, byte size);		// set current input buffer

#if DEBUG
void showRadioState();
#endif

#if USE_FCC_TESTING
void startRadioCW();								// start CW mode (for tesing)
void stopRadioCW();								// stop CW mode
#endif

#endif//__RADIO_H

