/*
 * File: Koliada - target.h
 *
 * Contains: CC8051 target defines
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
// map to the apporiate system support headers
#ifndef __TARGET_H
#define __TARGET_H
#include <stddef.h>

#include "compiler.h"
#include "typedefs.h"
#include "dataseg.h"
#include "debug.h"
#include "bios.h"
#include "part.h"

#include "assert.h"

// system specific >data< values and attributes

//#define USE_MODULES 1	// module managment
//#define USE_CMDLINE	1	// cmdline extensions
//#define USE_CMDTABLE	1	// extensible cmd table
#ifndef USE_MODULE_DATA
//	#define USE_MODULE_DATA 1
#endif

#define BOARD_CONFIG_HEADER	/* constant across all architectures                 */\
	char   boardName[32];		/* board label                                       */\
	UInt16 magic;					/* board data magic #                                */\
	UInt32 build;					/* structure version                                 */\
	UInt16 cksum;        	   /* checksum of this struct                           */\
	UInt16 major;					/* board major number                                */\
	UInt16 minor;					/* board minor number                                */\
	UInt16 config;					/* board configuration id                            */\
	UInt16 system;					/* system config bits                                */\
	\
	/* */

#if 0
/////////////////////////////////////////////////////////////////////
#define kMaxFlashRegions 8
typedef UInt16 FLASH_PTR;
typedef struct _CFI
	{
	// abstract version of struct defined in sysFlash.c
	// exposed by the target module for use by other system modules
	UInt8 *base;						// base of config flash
	UInt16 manufId;					// Manufacturer Id
	UInt16 devicId;					// Flash part ID
	UInt16 type;						// type of flash device
	UInt32 size;						// size of flash device
	UInt16 wbsize;						// write buffer depth
	UInt16 cmdSet;						// Intel or AMD
	UInt32 eraseTimeoutMs;			// Max erase timeout in milliseconds
	UInt32 writeTimeoutUs;			// Max write timeout in microseconds
	bool vpp;							// true if need vpp
	bool x16;							// width of device
	bool bootLow;						// boot block low/high
	UInt32 regions;					// number of erase block regions
	struct								// region map
		{
		UInt32 count;					// number of blocks in region
		UInt32 size;					// size of each block
		} map[kMaxFlashRegions];

	// flash vtable
	// raw flash i/o is not thread safe - low level module access only
	// it is assumed that flash is managed by a thread safe system library or task
	int (*write)(FLASH_PTR loc, int offset, FLASH_PTR src, unsigned count);
	int (*erase)(void *loc);				// must be erasable block address
	int (*lock)(void *loc, bool mode);	// mode = 0 - unlock, mode != 0 - lock
	} _CFI;
#define __CFI (*((_CFI **) FLASHCFI_ADDR))
#endif

#define kNumIVEC		18 // device specific??
#if 0
#define kNumShared	16
#define kNumMultix	16
#endif

#define kNumPTBL		8	// size of the pending object table
#define kNumCTBL		8	// size of the CLASS cache table

// system data segment
typedef struct SystemMemory
	{
	// system memory keeps track of various system params

	// since SystemMemory is loaded @ 0x0000, we define a null entry to detect null pointer refs
	word null;	// initialized to a 'special' value in initBoard()

	// Koliada ES supports IRQ events on CC25XX

	// Basic system info
	SYSPTR board;					// board configuration details

	// Here we expose a set of basic function tables. This set of functions is >not< designed to
	// be exhaustive! It is there primarily to support flash, basic i/o, gpio and other functions as
	// may be needed by all systems.
	//
	// These functions are garanteed thread safe and datasegment neutral and are are soft links that
	// may be replaced (or extended) by other sub-systems.
	//
	// The uart/gpio functions can (generally) support any UART/GPIO defined in the BoardConfig.

	SYSPTR uart;					// uart function table
	SYSPTR gpio;					// gpio function table
	SYSPTR fios;					// flash function table
	SYSPTR bios;					// pointer to the first bios link

	// System Object Model		// well known locations!!!!
	METHOD findMethod;			// entry to the class method dispatcher - returns method ptr
	METHOD callMethod;			// entry to the class method dispatcher - calls method and returns result

	FPTR strfmt;					// pointer to strfmt function
										// strfmt is big, it is made available here so modules don't have to drag it in

	// kernel exposes a system clock
	volatile UInt32 tickCount;	// current system tick count
	volatile UInt32 boot_time;	// seconds since 01/01/93 @ boot
	volatile UInt32 real_time;	// seconds since boot time
	volatile UInt32 real_tick; // ticks since real_time

	volatile word timerTicks;	// count down timer ticks (time since last expiration)
	volatile word timerCount;	// count down timer count
										// using 2kHz clock gives ~32s as largest timer interval!
	//
	UInt32 oldestStamp;			// oldest module that will work with this system
	UInt8  resetCause;			// set in board init (value is generally target specific)
	UInt8  resetCount;			// persists across a (warm) reset
	word pad1[3];					// pad to 16 byte boundary

	//
	OBJECT ITBL[kNumIVEC];		// interrupt object table
	OBJECT PTBL[kNumPTBL];		// pending objects table
	OBJECT CTBL[kNumCTBL];		// class table cache
	OBJECT VTBL[kNumIVEC];		// interrupt vector table

#if USE_MODULE_DATA
	// support for multiple load modules
	PTR moduleHead;				// head/tail of installed modules
	PTR moduleTail;
#endif

	OBJECT flash;					// flash object

#if USE_SHARED_COMPONENTS
	// support for shared APIs and Multix RTOS
	PTR sharedLib[kNumShared];	// shared system entry points
	PTR multixLib[kNumMultix];	// mpx (RTOS) system entry points
#endif

	volatile OBJECT oid;			// current 'control' object
	} SystemMemory;
extern SystemMemory sysData;
#define _sys (&sysData)

//#define BOARD ((BoardConfig *)_sys->board)
#define FLASH	(*((FlashTBL *)_sys->fios))
#define GPIO	(*((GpioTBL *)_sys->gpio))

// reset causes
enum
	{
	//
	// target genenric (may not all be avail in hardware)
	//
	kBrownoutReset = 1,	// power-on and brownout reset (is there a diff??)
	kExternalReset,		//
	kWatchdogReset,		//
	kSoftwareReset,		//
	kReservedReset = 0x15,
	//
	// target specific
	//
	};
void sysReboot();

// some well known locations
#define SYSTEM_TICKS	_sys->tickCount

// sys->ITBL object indicies - these remain constant (and unrelated to priority)
enum
	{
	/*  0 */ RFERR_ISR,   /*  RF TX FIFO Underflow and RX FIFO Overflow   */
	/*  1 */ ADC_ISR,     /*  ADC End of Conversion                       */
	/*  2 */ URX0_ISR,    /*  USART0 RX Complete                          */
	/*  3 */ URX1_ISR,    /*  USART1 RX Complete                          */
	/*  4 */ ENC_ISR,     /*  AES Encryption/Decryption Complete          */
	/*  5 */ ST_ISR,      /*  Sleep Timer Compare                         */
	/*  6 */ P2INT_ISR,   /*  Port 2 Inputs                               */
	/*  7 */ UTX0_ISR,    /*  USART0 TX Complete                          */
	/*  8 */ DMA_ISR,     /*  DMA Transfer Complete                       */
	/*  9 */ T1_ISR,      /*  Timer 1 (16-bit) Capture/Compare/Overflow   */
	/* 10 */ T2_ISR,      /*  Timer 2 (MAC Timer)                         */
	/* 11 */ T3_ISR,      /*  Timer 3 (8-bit) Capture/Compare/Overflow    */
	/* 12 */ T4_ISR,      /*  Timer 4 (8-bit) Capture/Compare/Overflow    */
	/* 13 */ P0INT_ISR,   /*  Port 0 Inputs                               */
	/* 14 */ UTX1_ISR,    /*  USART1 TX Complete                          */
	/* 15 */ P1INT_ISR,   /*  Port 1 Inputs                               */
	/* 16 */ RF_ISR,      /*  RF General Interrupts                       */
	/* 17 */ WDT_ISR,     /*  Watchdog Overflow in Timer Mode             */
	};

// abstract remapping
#define CMTIMER_ISR T2_ISR
#define uartRx_ISR URX1_ISR
#define uartTx_ISR UTX1_ISR

enum sysGpio
	{
	kSysGpio0,
	kSysGpio1,
	kSysMaxGpio,
	};
#define kSysGpio kSysGpio0

enum
	{
	_kGpioModeI,	// in
	_kGpioModeO,	// out
	};

enum
	{
	_kGpioLo,	// 0
	_kGpioHi,
	};

// defined system LEDs
#define kSysLed 0	// always pin 0

#if USE_CMDLINE
	// we use one particular memory location to buffer cmd args
	#define CMDLINE_ADDR		((void *) 0x2000e000)
	#define ARGV_ADDR			((void *) 0x2000f000)
	#define kMaxCmdline		1024
	#define kMaxArgs			256	// including NULL = 0x1000(4096) bytes (~4 cmdline bytes/arg)
	// ends at 0x0x08009400
#endif

/////////////////////////////////////////////////////////////////////

// shareApi va_args
#define SHARE_ARGS_DECL(v)\
	unsigned *_args = &v;_args++;

// args[4-8] are the swi interlude but are (currently) disambiguated in svcDispatch
#define SHARE_ARGS_LIST _args[0], _args[1], _args[2], _args[3], _args[4], _args[5], _args[6], _args[7]
#define SHARE_ARGS _args
#define SHARE_ARG0 _args[0]
#define SHARE_ARG1 _args[1]
#define SHARE_ARG2 _args[2]
#define SHARE_ARG3 _args[3]
#define SHARE_ARG4 _args[4]
#define SHARE_ARG5 _args[5]
#define SHARE_ARG6 _args[6]
#define SHARE_ARG7 _args[7]

////////////////////////////////////////////////////////////////////

#ifndef assert
	#ifdef NDEBUG
		#define assert(ignore)((void) 0)
	#else
		#if !RELEASE
			#define assert(e) ((e) ? 0 : debug("%s.%d: %s assert failed\n", __FILE__, __LINE__, #e))
			// define this before OR redefine it after
			// this edition does not call abort since the exception handling is not ported to this version yet
		#else
			#define assert(ignore)
		#endif
	#endif
#endif

#define bitset(a, b, c) (a = (c) ? (a)|(1<<(b)) : (a)&~(1<<(b)))
#define isbitset(a, b)	((a) & (1<<(b)) ? 1 : 0)
#define test(a, b)		(((a) & (b)) ? 1 : 0)

#define __C	// C entry point

#define _sysGetTickCount() _sys->tickCount

// 8051
#define DUMP_STACK()
#define waitAtomic()\
	/* disable interrupts */      \
	__asm("PUSH 0xA8");/* IE */   \
	__asm("CLR  0xAF");/* IE.7 */
#define postAtomic()\
	__asm("POP 0xA8"); /* IE */

#endif//__TARGET_H
