/*
 *	File: Koliada - uart.h
 *
 *	Contains: basic, non blocking, no interrupt, uart i/o for use during bootstrap/debug
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2012 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#ifndef __UART_H
#define __UART_H

// system only defines one uart - any others are down to you!

// UART descriptor
typedef struct UartDescriptor
	{
	// uart register addresses
	UInt8 csr;		// control & status
	UInt8 ucr;		// uart control
	UInt8 gcr;		// generic control
	UInt8 buf;		// i/o buffer
	UInt8 baud;		// baud rate control
	//
	struct
		{
		UInt8 PERCFG : 1;	// alternate config
		UInt8 BAUD_E : 7; // baud exponent
		};
	UInt8 BAUD_M;
	} UartDescriptor;

typedef struct UartConfig
	{
	byte count;						// number of uarts
	UartDescriptor *uartTable;	// uart config data
	} UartConfig;

// UartTBL	                     // uart function table
typedef CONST struct UartTBL
	{
	int (*putc)(int c);				// putc function
	int (*poll)();						// poll function (does not block)
	int (*getc)();						// getc function (does not block)
	int (*getk)(word ms);			// getc function (w. timed wait)
	int (*read)(byte *buf, byte len);	// read function (buf must be static)
	int (*write)(byte *buf, byte len);	// write function (buf must be static)
//	void (*setBuffer)(char mode, byte *buf, byte len);
//	void (*getBuffer)(char mode, byte **buf, byte *len);
	} UartTBL;
#define UART	(*((UartTBL *)_sys->uart))

// UartTBL is not extensible, but it is replacable via _sys->uart
// replacements must match this vtable!

int uartPutc(int c);
int uartGetc();
int uartGetkey(word ms);
int uartRead(byte *buf, byte len);
int uartWrite(byte *buf, byte len);

void uartSetBuffer(char mode, byte *buf, byte len);
void uartGetBuffer(char mode, byte **buf, byte *len);

#endif//__UART_H
