/*
 * lis2dh.c
 *
 *  Created on: Jan 24, 2015
 *      Author: shaun.astar@gmail.com
 */

#include "spi.h"
#include "lis2dh.h"
#include "lpm.h"

#define DATA_RATE		3			// ODR 2=10Hz, 3=25Hz, 4=50Hz See Table 28


void lis2dh_init(void)
	{
	volatile unsigned char data;

	lis2dh_gpio();

	spiTx(LIS2DH_CTRL_REG1, (DATA_RATE << 4) + 0x04);	// normal mode Z-axis enabled
	spiTx(LIS2DH_CTRL_REG2, 0x00);		// HPF disabled
	spiTx(LIS2DH_CTRL_REG3, 0x40);		// INT1=AOI1
	spiTx(LIS2DH_CTRL_REG4, 0x00);		// FS=+/-2G,4-wire SPI
	spiTx(LIS2DH_CTRL_REG5, 0x05);		// No FIFO, normal mode
	spiTx(LIS2DH_CTRL_REG6, 0x00);		// INT2=Click, active high - disabled
//	spiTx(LIS2DH_CTRL_REG6, 0xa0);		// INT2=Click, active high

	// INT1 used for dynamic orientation detection: Top/Side/Upside-down
	spiTx(LIS2DH_INT1_THS, 0x1f);		// 32LSB*15.625mg/LSB=0.5g
	spiTx(LIS2DH_INT1_DURATION, 0x05);	// Duration= 5LSB*(1/10Hz) = 500ms, or =200ms@25Hz
	spiTx(LIS2DH_INT1_CFG, 0xf0);		// Direction detection on Z-axis zone

#if 0
	// INT2 used for wake up event from low power mode
	spiTx(LIS2DH_INT2_THS, 0x3f);		// 64*15.625mg/LSB=1g shock
	spiTx(LIS2DH_INT2_DURATION, 0x00);	// Duration= 0LSB*(1/10Hz) = 0 sec
	spiTx(LIS2DH_INT2_CFG, 0xf0);		// when Z axis outisde zone

	spiTx(LIS2DH_TIME_LIMIT, 0x01);		// outside zone THS > 100ms
	spiTx(LIS2DH_TIME_LATENCY, 0x00);	// INT2 delay after event
	spiTx(LIS2DH_TIME_WINDOW, 0x0a);	// between two clicks 0x0a=1sec
	spiTx(LIS2DH_CLICK_THS, 0x3f);		// zone +/- threshold 0x3f=1g
	spiTx(LIS2DH_CLICK_CFG, 0x20);		// double flip on Z axis
	spiTx(LIS2DH_CLICK_SRC, 0x64);		// double flip on Z axis enable
#endif
	}

unsigned char lis2dh_getId(void)
	{
	volatile unsigned char id;
	id = spiRx(LIS2DH_WHO_AM_I);
	return id;
	}

int lis2dh_getZ(void)
	{
	int data;

	data   = spiRx(LIS2DH_OUT_Z_H);
	data <<= 8;
	data  += spiRx(LIS2DH_OUT_Z_L);

	return data;
	}

int lis2dh_getClickSrc(void)
	{
	unsigned char data;

	data   = spiRx(LIS2DH_CLICK_SRC);
	return data;
	}

int lis2dh_getInt2Src(void)
	{
	unsigned char data;

	data   = spiRx(LIS2DH_INT2_SOURCE);
	return data;
	}

#if 0	// ISR in lpm.c
// Port 2 interrupt service routine
#pragma vector=PORT2_VECTOR
__interrupt void Port_2(void)
	{
	volatile unsigned char request, int2Src, clickSrc;                // interrupt request

	// all falling edge
	request = P2IFG & (ACCINT2 | ACCINT1); 			// copy of interrupts

	if (request & ACCINT1)
		{
		// Z-axis significant change
		//P1OUT ^= 0x01;                           // P1.0 = toggle
		LPM0_EXIT;
		}

	if (request & ACCINT2)
		{
		// Button
		//P1OUT ^= RGB_B;                         // P1.0 = toggle
		//int2Src = list2dh_getInt2Src();
		clickSrc = list2dh_getClickSrc();
		if (clickSrc & (LIS2DH_CLK_IA_MASK + LIS2DH_DCLICK_MASK))
			{
			flags |= FLAG_SHOCK;						// shock detected
			LPM0_EXIT;
			}
		}

	P2IFG &= ~(ACCINT2 | ACCINT1);   // IFG cleared
	}

#endif

