#include "TCA6416A.h"
#include "hal_i2c.h"
#include "sysgpio.h"
#include "led.h"


//void buttonEvent(byte button);
void accEvent(byte acc);
void ledBlink(Pattern pattern);
int printf(const char *, ...);
void checkExpander(void);
void button3Event();
#define EXPANDER_INT 15
/*
TCA6416A IO Expander Driver

MeshCandy Badge
Board: TENA006
P00 = LED_ON (output)
P01 = SENSOR_ON (OUTPUT)
P02 = SW3 (input)
P03-P07 NOT CONNECTED
P10 = NOTHING (output)
P11 = NOTHING (output)
P12 = NOTHING (output)
P13 = NOTHING (output)
P14 = GYRO_ON (output)
P15 = GYRO_ON (output)
P16 = GYRO_INT1 (input)
P17 = GYRO_INT2 (input)
*/

static byte wake = 0;
static SysPowerMode(*chainEventSleepNotice)(SysPowerMode);
SysPowerMode eventSleepNotice(SysPowerMode mode)
	{
#if DEBUG
	static const char *const *modes =
		{
		"kSysWake",
		"kSysSnooze",
		"kSysSleep",
		"kSysDown"
		};
	assert(mode < 4);
	debug("wake=%u: %s\n", modes[(wake ? kSysWake : mode)]);
#endif
	return chainEventSleepNotice(wake ? kSysWake : mode);
	}

void waitExpander()
	{
#if DEBUG
	if (!(wake < 32))
		debug("%s: suspicious wake value - %u!\n", __FUNC__, wake);
#endif
	wake++;
	}

void postExpander()
	{
#if DEBUG
	if (wake == 0)
		debug("%s: bad wake value - %u!\n", __FUNC__, wake);
	else
#endif
	wake--;
	}

byte readExpander(byte addr)
	{
	selectI2C(TCA6416A_ADDR, i2cClock_267KHZ); // select IO expander as I2C slave
	return i2cRx(addr);
	}

void writeExpander(byte addr, byte value, UInt8 nBytes)
	{
	selectI2C(TCA6416A_ADDR, i2cClock_267KHZ); // select IO expander as I2C slave
	i2cWriteReg(addr, &value, nBytes);
	}

DefineEvent(expanderEvent);
void expanderEventHandler(EVENT e, word ifg, word ier)
	{
	byte Port0 = 0;
	byte Port1 = 0;
	byte button_int = 0;
	byte gyro_int = 0;

	if (ifg & 0x80)
		{
		//selectI2C(TCA6416A_ADDR, i2cClock_267KHZ); // select IO expander as I2C slave
		//ledBlink(redBlinkPattern);
		Port0 = readExpander(PORT0_IN); // read Input Port 0 register
		Port1 = readExpander(PORT1_IN); // read Input Port 1 register
		//IEN2 |= 0x02;

		button_int = (~Port0 & (BUTTON_3));
		gyro_int = (~Port1 & (GYRO_INT1 + GYRO_INT2));
		if (button_int) // If one of the buttons were triggered
			{
			button3Event();  // high bit signifies button press
			}
		if (gyro_int)
			{
			//accEvent(gyro_int);
			GPIO.IFG(EXPANDER_INT, 0);
			GPIO.IEN(EXPANDER_INT, 1);
			}
		}
	}


/*
Initialize IO Expander
*/
void initExpander(void)
	{
	byte Port0;
	byte Port1;
	chainEventSleepNotice = setEventSleepNotice(eventSleepNotice);

	selectI2C(TCA6416A_ADDR, i2cClock_267KHZ); // select IO expander as I2C slave
	i2cTx(PORT0_CFG, BUTTON_3);
	i2cTx(PORT1_CFG, GYRO_INT1 + GYRO_INT2);

	//i2cTx(PORT1_INV, ACC_INT1 + ACC_INT2);
	//i2cWriteReg(PORT0_CFG, &Port0, 1); // configure P00, P01, P02 as inputs, the rest as outputs
	//i2cWriteReg(PORT1_CFG, &Port1, 1); // Configure P17, P16 as inputs, rest outputs

	i2cTx(PORT1_OUT, /*GYRO_ON +*/ GYRO_INT1 + GYRO_INT2);
	i2cTx(PORT0_OUT, (BUTTON_3) + SENSOR_ON);

	Port0 = readExpander(PORT0_IN); // read Input Port 0 register
	Port1 = readExpander(PORT1_IN); // read Input Port 0 register
#if 0
	printf("Port 0 CFG: %x\n", Port0);
	printf("Port 1 CFG: %x\n", Port1);
#else
	// silence compiler
	(void) Port0;
	(void) Port1;
#endif
	// create a button event
	objectCreate(expanderEvent);
	OnEvent(expanderEvent, (HANDLER) expanderEventHandler);

	// capture BUTTON interrupt to triger the event
	// note: it is up to the developer to manage overlapping pin port IRQs!
	setEventIRQ(PORT_IRQ(PIN_PORT(EXPANDER_INT)), expanderEvent);

	GPIO.IFG(EXPANDER_INT, 0);
	GPIO.IEN(EXPANDER_INT, 1);
	}
