#include "EtherDATA.h"

#include "dbData.ddl.h"
#include "sysGpio.h"
#include "led.h"

int printf(const char *, ...);
void buttonPressed(int button);
void buttonHeld(int button);
void ledOn(word led, word delay);
#define kHoldTime TICKS(1200)
#define kPressTime TICKS(200)

#define BUTTON 3	// virtual BUTTON pin #
#define kDebounceDelay TICKS(1)

DefineTimer(buttonHeldTimer);
void buttonHeldHandler(TIMER t, int button)
	{
//	debug("%s\n", __func__);
	ledOn(LED_RED, 200);
	buttonHeld(button);
	}


DefineTimer(buttonTimer);
void buttonTimerHandler(TIMER t, int button)
	{
//	debug("%s\n", __func__);
	if (GPIO.GET(button) == 0)
		{
		// button is still pressed - see if it is a 'hold'
		cmSetTimer(buttonHeldTimer, kHoldTime); 	// reset and start the button hold timer
		cmStartTimer(buttonHeldTimer, button);		// add the button as an arg
		}
	else
		{
		// must be a press
		buttonPressed(button);
		}

	GPIO.IFG(button, 0);
	GPIO.IEN(button, 1);
	}



DefineEvent(buttonEvent);
void buttonEventHandler(EVENT e, word ifg, word ier)
	{
	// called each time the BUTTON is pushed
//	debug("%s\n", __func__);
	sysDelayMs(kDebounceDelay);
	if (GPIO.GET(BUTTON) == 0)
		{
		cmSetTimer(buttonTimer, kPressTime); 	// reset and start the button debuounce timer
		cmStartTimer(buttonTimer, BUTTON);		// add the button as an arg
		}
	else
		{
		GPIO.IFG(BUTTON, 0);
		GPIO.IEN(BUTTON, 1);
		}
	}

void initButton()
	{
	// button debounce timer
	objectCreate(buttonTimer, kSingleShotTimer, 0);
	OnEvent(buttonTimer, (HANDLER) buttonTimerHandler);

	// button held timer
	objectCreate(buttonHeldTimer, kSingleShotTimer, 0);
	OnEvent(buttonHeldTimer, (HANDLER) buttonHeldHandler);


	// create a button event
	objectCreate(buttonEvent);
	OnEvent(buttonEvent, (HANDLER) buttonEventHandler);

	// capture BUTTON interrupt to triger the event
	// note: it is up to the developer to manage overlapping pin port IRQs!
	setEventIRQ(PORT_IRQ(PIN_PORT(BUTTON)), buttonEvent);

	// enable interrupts for the buttons
	GPIO.IFG(BUTTON, 0);
	GPIO.IEN(BUTTON, 1);
	}