/*
 * File:
 *
 * Contains:
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 *  Copyright  1989-2014 by Guy McIlroy
 *  All rights reserved.
 *
 *  This module contains confidential, unpublished, proprietary source code.
 *  The copyright notice above does not evidence any actual or intended
 *  publication of such source code.
 *
 *  This code may only be used under licence from Koliada, LLC - www.koliada.com
 *
 * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * * *
 *
 */
#include <stdio.h>
#include <string.h>

#include "Koliada.h"
#include "timer.h"
#include "sysGpio.h"

#include "pCache.h"
#include "EtherDATA.h"
#include "dbData.ddl.h"

#include "subsSave.h"

#define LED_RED 0	// virtual LED pin
#define LED_YEL 1
#define LED_GRN 2

void rssiGetHandler(string remote);
void rssiGet(dbValue ref, string group);
int sCompare(void *key, void *buf, SIZE_T size);
int uCompare(void *key, void *buf, SIZE_T size);

enum
	{
	kDdbGet = 1,
	kDbSet,
	kDbGetValue,
	kDbSetValue,
	kDbSubscribe,
	kDbUnsubscribe,
	kEmNeighborTable,
	kEmResolveName,
	kEmResolveAddr,
	kDbRename,
	kDbSetArrayValue,
	};
char *fnames[] =
	{
	/*  0 */ "invalid",
	/*  1 */ "dbGet",
	/*  2 */ "dbSet",
	/*  3 */ "dbGetValue",
	/*  4 */ "dbSetValue",
	/*  5 */ "dbSubscribe",
	/*  6 */ "dbUnsubscribe",
	/*  7 */ "emNeighborTable",
	/*  8 */ "emResolveName",
	/*  9 */ "emResolveAddr",
	/* 10 */ "dbRename",
	/* 11 */ "dbSetArrayValue",
	};
#define sizeofFnameTable (sizeof(fnames)/sizeof(char *))

#include <stdarg.h>
#include "strfmt.h"
unsigned atoh(const char *);
int stricmp(const char *, const char *);
int subCounter;

byte uartPuts(byte *buf);

static char txBuf[128];
static byte putIndex;
static bool newSend = true;
static byte count;

void sendTxBuf()
	{
	  printf("sendTxBuf start...\n");
	GPIO.XOR(0); // turn light on when sending data through serial
	printf("sendTxBuf GPIO.XOR...\n");

	debug("%s%s", newSend ? "<< " : "", txBuf);
	uartPuts((byte *) txBuf);
	printf("uartsPuts...\n");

	newSend = false;
	printf("newSend...\n");
	putIndex = 0;
	count = 0;
	
	GPIO.XOR(0); // turn light off when finished
	printf("GPIO.XOR...\n");
	printf("sendTxBuf end...\n");

	}

static int outc(int c)
	{
	txBuf[putIndex++] = c;
	txBuf[putIndex] = 0;
	if (c == '\n' || !(putIndex < sizeof(txBuf) - 1))
		{
		sendTxBuf();
		if (c == '\n')
			newSend = true;
		}
	return c;
	}

int output(const char *fmt, ...)
	{
	// response output handler
	register unsigned i;
	va_list args;

	va_start(args, fmt);

	outch_t outch;
	outch.out = outc;
	i = STRFMT(outch, 0, fmt, args);

	va_end(args);
	return i;
	}

static word function(char *fn)
	{
	for (byte i = 1; i < sizeofFnameTable; i++)
		if (strcmp(fnames[i], fn) == 0)
			return i;

	debug("%s: cmd not found!!\n", fn);
	return 0;
	}

#if SHOW_ARRAYS
static STATUS arrayHandler(dbValue ref, void *cookie, int index, Int8 *value)
	{
	string ident;
	dbGetName(ref, &ident);

	printf("%s.%s[%u]: %d\n", cookie, ident, index, *value);

	return kScanContinue;
	// return value has actions
	// 0 - continue scan
	// 1 - stop scan
	}

static void arrayCallback(string group, dbValue ref)
	{
	printf("%s\n", __func__);

	// iterate through the ref array
	dbScanArray(ref, group, arrayHandler);

	printf("\n");
	}
#endif

static STATUS arrayHandler(string group, dbValue ref, void *cookie, byte index, void *value) // if dataobject is an array
	{
	string ident;
	byte type;
	dbGetName(ref, &ident);
	dbGetType(ref, &type);
	printf("arrayHandler\n");
	if (index)
		putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, ",");

	//putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d",data[index]);
	dbGetType(ref, &type);
	switch (type)
		{
		case TYPE_INT8:
				{
				Int8 data[kDbValueCount];
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d", data[index]);
				break;
				}
		case TYPE_CHAR:
				{
				string data;
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%s", data);
				}
		case TYPE_BYTE:
		case TYPE_UINT8:
				{
				UInt8 data[kDbValueCount];
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d", data[index]);
				break;
				}

		case TYPE_INT:
				{
				printf("TYPE_INT start\n");
				int data[kDbValueCount];
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d", data[index]);
				printf("TYPE_INT end\n");

				break;
				}
		case TYPE_WYDE:
		case TYPE_WORD:
		case TYPE_INT16:
				{
				Int16 data[kDbValueCount];
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d", data[index]);
				break;
				}
		case TYPE_UINT16:
				{
				UInt16 data[kDbValueCount];
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%d", data[index]);
				break;
				}

		case TYPE_STRINGREF:
		case TYPE_STRINGBUF:
				{
				string data;
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "%s", data);
				break;
				}

		case TYPE_BOOL:
		case TYPE_TETA:
		case TYPE_INT32:
		case TYPE_UINT32:
			// fall thru
		case TYPE_REAL:
		default:
			debug("unsupported value type!\n");
			break;
		}

	if (index == (count - 1))
		{
		printf("if index == (count-1) start\n");

		putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "]}\n");
		sendTxBuf();
		printf("if index == (count-1) end\n");

		return kScanStop;
		}

	return kScanContinue;
	// return value has actions
	// 0 - continue scan
	// 1 - stop scan
	}

void callback(string group, dbValue ref, void *value) // CALLBACK for all data objects!
	{
	string ident;
	dbGetName(ref, &ident);
	byte type;
	dbGetType(ref, &type);
	printf("CALLBACK\n");
#if SHOW_ARRAYS
	byte count;
	dbGetAttribute(ref, kDbValueCount, &count);
	if (count)
		{
		debug("%s.%s is an array of %u elements: ", group, ident, count);
		printf("go to arraycallback...\n");
		arrayCallback(group, ref);
		}
#endif
	dbGetAttribute(ref, kDbValueCount, &count);

	switch (type)
		{
		case TYPE_INT8:
		case TYPE_CHAR:
				{
				static char raw[18];
				static char data[18];
				//string test;
				memset(raw, 0, sizeof(raw));
				memset(data, 0, sizeof(data));
				dbGet(ref, &raw);
				memcpy(data, raw, sizeof(data));
				//memcpy(test, data, sizeof(data));
				putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":\"%s\"}\n", group, ident, data);
				sendTxBuf();
				break;
				}

		case TYPE_BYTE:
		case TYPE_UINT8:;
			UInt8 data8;
			dbGet(ref, &data8);
			putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":%d}\n", group, ident, data8);
			sendTxBuf();
			break;

		case TYPE_INT:
		case TYPE_WYDE:
		case TYPE_WORD:
		case TYPE_INT16:
		case TYPE_UINT16:
			if (count)
				{
				putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":[", group, ident);
				// iterate through the ref array
				dbScanArray(ref, group, arrayHandler);
				}
			else if (type == TYPE_UINT16)
				{
				UInt16 data16;
				dbGet(ref, &data16);
				putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":%x}\n", group, ident, data16);
				sendTxBuf();
				//printf("{\"group\":\"%s\",\"%s\":%d}\n", group, ident, data);
				}
			else
				{
				UInt8 data8;
				dbGet(ref, &data8);
				putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":%d}\n", group, ident, data8);
				sendTxBuf();
				}
			break;

		case TYPE_STRINGREF:
		case TYPE_STRINGBUF:
				{
				UInt16 data;
				dbGet(ref, &data);
				putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":\"%s\"}\n", group, ident, data);
				sendTxBuf();
				}
			break;

		case TYPE_BOOL:
		case TYPE_TETA:
		case TYPE_INT32:
		case TYPE_UINT32:
		case TYPE_REAL:
		default:
#if 1
			if (strcmp(ident, "rssiTable") == 0) // if neighbortable, do something special
				{
				rssiGet(ref, group);
				debug("rssiTable returned\n");
				break;
				}
			putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":[", group, ident);
			// iterate through the ref array
			dbScanArray(ref, group, arrayHandler);
			break;
#endif
			// fall thru
			debug("unsupported value type!\n");
			break;
		}

	// encode value into a buffer as {"group":"<group>","<ident>":"<value>"}
	/*putIndex = strnbuf(sizeof(txBuf), txBuf, 0, "{\"group\":\"%s\",\"%s\":", group, ident);
	//putIndex = dbEncode(kStringEncoding, ref, sizeof(txBuf), (byte *) txBuf, putIndex);
	putIndex = strnbuf(sizeof(txBuf), txBuf, putIndex, "}\n");

	// put the result to the uart
	sendTxBuf();*/
	}

void *decodeValue(dbValue ref, char *value)
	{
	static byte data[8];
	byte type;
	void *ptr;

	// decode the value to set
	dbGetType(ref, &type);
	switch (type)
		{
		case TYPE_INT8:
		case TYPE_CHAR:
		case TYPE_BYTE:
				{
				ptr = value;
				break;
				}
		case TYPE_UINT8:
			*((byte *)(ptr = data)) = atoi(value);
			break;

		case TYPE_INT:
		case TYPE_WYDE:
			//#warning depends on word size!!
		case TYPE_WORD:
		case TYPE_INT16:
		case TYPE_UINT16:
			*((word *)(ptr = data)) = atoi(value);
			break;

		case TYPE_BOOL:
			if (stricmp((const char *) "true", (const char *) value) == 0)
				*((bool *)(ptr = data)) = TRUE;
			else
				*((bool *)(ptr = data)) = false;
			break;

		case TYPE_STRINGREF:
		case TYPE_STRINGBUF:
			// by value
			ptr = value;
			break;

		case TYPE_TETA:
		case TYPE_INT32:
		case TYPE_UINT32:
			// fall thru
		case TYPE_REAL:
		default:
			debug("unsupported value type!\n");
			ptr = NULL;
			break;
		}

	return ptr;
	}

#define groupArg	argv[0]
#define identArg	argv[1]	// ident is the name of the dbValue which is being operated on.
#define valueArg	argv[2]	// value is a string encoding of the value we want to assign to the dbValue (ref);
#define indexArg	argv[3]	// index is the index into a dbValue array

void exec(char *fn, byte argc, char **argv, va_list args) // execute the command
	{
	void *ptr;
	STATUS err;
	dbValue ref;
	emAddr addr;
	string ident;

#if 1
	debug("== %s(", fn);
	for (byte i = 0; i < argc; i++)
		debug("%s, ", argv[i]);
	debug("%s)\n", argc ? "\b\b" : "");
#endif

	// reset the txBuf index
	putIndex = 0;

	switch (function(fn))
		{
		case 0: // invalid
			error("%s: undefined function!\n", fn);
			break;

		case kDbRename:
			if (dbRename(addr = strtol(argv[0], NULL, 16), identArg))
				{
				error("%x: rename as \"%s\" failed!\n", addr, identArg);
				return;
				}
			output("{\"dbRename\":%x,\"name\":\"%s\"}\n", addr, identArg);
			dbSetValue(identArg, led, 1); // send confirmation
			break;

		case kDbSetValue: // sets a named value in a specific group (tag - A,B,C,D,E)
			if (dbFindValue(identArg, &ref))
				{
				error("%s: undefined value!\n", identArg);
				return;
				}
			if ((ptr = decodeValue(ref, valueArg)) == NULL)
				{
				// bad value decode
				error("%s: bad value decode!\n", valueArg);
				return;
				}
			err = dbSetValueByRef(groupArg, ref, ptr);
			break;

		case kDbSetArrayValue: // sets a named value array item in a specific group (tag - A,B,C,D,E)
			if (dbFindValue(identArg, &ref))
				{
				error("%s: undefined value!\n", identArg);
				return;
				}
			if ((ptr = decodeValue(ref, valueArg)) == NULL)
				{
				// bad value decode
				error("%s: bad value decode!\n", valueArg);
				return;
				}
			err = dbSetArrayValue(groupArg, ref, atoi(indexArg), ptr);
			break;

		case kDbGetValue: // gets a named value from a specific group (tag - A,B,C,D,E)
			if (dbFindValue(identArg, &ref))
				{
				error("%s: undefined value!\n", identArg);
				return;
				}
			err = dbGetValue(groupArg, ref, NULL, 5);
			if (err)
				break;

			// 'call back' the result
			callback(groupArg, ref, NULL);
			break;

		case kDbSubscribe: // subscribe to a value
			if (dbFindValue(identArg, &ref))
				{
				printf("%s: undefined value!\n", identArg);
				return;
				}
			//memcpy(&sTable->id, &subCounter, sizeof(subCounter));
			if (!dbSubscribe(groupArg, ref, callback)) // this is where stations save USB subscriptions in PCACHE
				{
				struct SUBR sTable, *test = &sTable;
				strnbuf(20, (test = &sTable)->groupName, 0, "%s", groupArg);
				memcpy(&(test = &sTable)->groupObject, &ref, sizeof(ref));
				newChunk(kSubscribeType, &sTable, 20 + sizeof(ref));
				printf("sizeof chunk: %d\n", strlen(groupArg) + sizeof(ref));
				}

			//delChunk(kSubCounterType, NULL, delCompare);
			//newChunk(kSubCounterType, &subCounter, sizeof(subCounter));
			//subCounter += 1;
			//locChunk(kSubscribeType, NULL, sCompare);
			break;

		case kDbUnsubscribe: // unsubscribe from a value
			if (dbFindValue(identArg, &ref))
				{
				error("%s: undefined value!\n", identArg);
				return;
				}
			dbUnsubscribe(groupArg, ref); // if unsubscribe is called, delete the subscription from PCACHE
			struct SUBR uTable, *utest = &uTable;
			strnbuf(20, (utest = &uTable)->groupName, 0, "%s", groupArg);
			memcpy(&(utest = &uTable)->groupObject, &ref, sizeof(ref));
			delChunk(kSubscribeType, &uTable, uCompare);
			break;

		case kEmNeighborTable: // get a node neighbor table
			rssiGetHandler(groupArg);
			break;

		case kEmResolveName: // resolve address as name
			ident = emResolveName(addr = atoi(argv[0]));
			if (ident == NULL)
				output("{\"emResolvedName\":null,\"addr\":%u}\n", addr);
			else
				output("{\"emResolvedName\":\"%s\",\"addr\":%u}\n", ident, addr);
			break;

		case kEmResolveAddr: // resolve name as address
			addr = emResolveAddr(ident = argv[0]);
			if (addr == (word) - 1)
				output("{\"emResolvedAddr\":\"%s\",\"addr\":null}\n", ident);
			else
				output("{\"emResolvedAddr\":\"%s\",\"addr\":%x}\n", ident, addr);
			break;
		}
	}

