// MS8607 PTH sensor driver

#include "PHT Drivers.h"
#include "hal_i2c.h"
#include "pindefs.h"

#include "string.h"
#include "math.h"

#include "TCA6416A.h"

// Sensor I2C address
#define SENSOR_HUMIDITY_I2C_ADDRESS      0x40
#define SENSOR_PRES_TEMP_I2C_ADDRESS     0x76

#define S_REG_LEN                  2
#define DATA_LEN                   3
#define PRES_TEMP_LEN              8

/* Sensor bit values for self-test and DCDC conbtrol */
#define ST_IRTEMP                             0x01
#define ST_HUMID                              0x02
#define ST_ACC                                0x08
#define ST_GYRO                               0x20

#define HUMIDITY  1
#define PRES_TEMP 2

#define HIGH_SUPPLY_SENSOR_MAP                ( ST_IRTEMP | ST_HUMID | ST_GYRO)

// PRESSURE AND TEMP COMMANDS
#define SENSOR_PRES_TEMP_RESET             0x1E
#define SENSOR_PRES_256            0x40
#define SENSOR_PRES_512            0x42
#define SENSOR_PRES_1024           0x44
#define SENSOR_PRES_2048           0x46
#define SENSOR_PRES_4096           0x48
#define SENSOR_PRES_8192           0x4A
#define SENSOR_TEMP_256            0x50
#define SENSOR_TEMP_512            0x52
#define SENSOR_TEMP_1024           0x54
#define SENSOR_TEMP_2048           0x56
#define SENSOR_TEMP_4096           0x58
#define SENSOR_TEMP_8192           0x5A
#define SENSOR_PRES_TEMP_ADC_READ          0x00
#define SENSOR_PROM_READ             0xA0 // NOT SURE?

// Internal commands
#define SENSOR_HUMID_H                  0xE5 // command trig. humidity meas. hold master
#define SENSOR_HUMID_NH                 0xF5 // command trig. humidity meas. no hold master
#define SENSOR_HUMID_WRITE_USER         0xE6 // command write user register
#define SENSOR_HUMID_READ_USER          0xE7 // command read user register
#define SENSOR_HUMID_RESET              0xFE // command soft reset

#define USR_REG_MASK               0x38  // Mask off reserved bits (3,4,5)
#define USR_REG_DEFAULT            0x02  // Disable OTP reload
#define USR_REG_RES_MASK           0x7E  // Only change bits 0 and 7 (meas. res.)
#define USR_REG_11BITRES           0x81  // 11-bit resolution

/* ------------------------------------------------------------------------------------------------
*                                           Type Definitions
* ------------------------------------------------------------------------------------------------
*/

/* ------------------------------------------------------------------------------------------------
*                                           Local Functions
* ------------------------------------------------------------------------------------------------
*/

//static void Sensor_Pwr(bool option);
//static void HalHumiSelect(void);
void Sensor_Select(int sensor);
static bool Humi_Read_Data(unsigned long *pBuf, UInt8 nBytes);
static bool Sens_Read_Data(unsigned long *pBuf, UInt8 nBytes);
//static bool HalHumiReadData(UInt8 *pBuf,UInt8 nBytes);
static bool Sens_Humi_Write_Cmd(UInt8 cmd);
int printf(const char *, ...);

/* ------------------------------------------------------------------------------------------------
*                                           Local Variables
* ------------------------------------------------------------------------------------------------
*/
static unsigned long usr;                         // Keeps user register value
static unsigned long buf[8];                      // Data buffer
static unsigned long pres_buf[3];
static unsigned long temp_buf[3];
static double dT = 0;
static double T = 0;
static double OFF = 0;
static double SENSE = 0;
static double Pressure = 0;
unsigned long raw_pres;
unsigned long raw_temp;
static bool  success;
// from hal_sensor.c
static UInt8 buffer[24];
ExternTimer(humiDelayTimer);
extern UInt8 D1[24];
extern UInt8 D2[24];
unsigned long C[8];

/*
Sensor_Pwr(bool option)
1 - ON
0 - OFF
*/
void Sensor_Pwr(bool option)
	{
	//GPIO.SET(SENSOR_PWR, option);
	}

/**************************************************************************************************
* @fn          SensorInit
*
* @brief       Initialise the humidity sensor driver
               Process defined in Sensor's datasheet
*
* @return      none
**************************************************************************************************/
void SensorInit(void)
	{
	// need the i/o expander to stay awake
	waitExpander();

	Sensor_Select(HUMIDITY);

	success = Sens_Humi_Write_Cmd(SENSOR_HUMID_RESET);
	cmDelayMs(15);

	// Set 11 bit resolution
	SensorReadReg(SENSOR_HUMID_READ_USER, &usr, 1);
	usr &= USR_REG_RES_MASK;
	usr |= USR_REG_11BITRES;
	SensorWriteReg(SENSOR_HUMID_READ_USER, &usr, 1);
	success = FALSE;

	// done with i/o expander
	postExpander();
	}


/**************************************************************************************************
* @fn          HalHumiExecMeasurementStep
*
* @brief       Execute measurement step
*
* @return      none
*/
bool HumiExecMeasurementStep(UInt8 state)
	{
	//HalI2CDisable();
	Sensor_Select(HUMIDITY);
	//cmDelayMs(5);

	/*success = Sens_Humi_Write_Cmd(SENSOR_HUMID_H);
	cmDelayMs(25);
	Sensor_Select(HUMIDITY);
	if (success)
	{
	  success = Humi_Read_Data(buf+DATA_LEN, DATA_LEN);

	}*/
	switch (state)
		{
		case 0:
				{
				// Start temperature read
				//success = HalHumiWriteCmd(HTU21D_CMD_TEMP_T_NH);
				//success = HalHumiWriteCmd(HTU21D_CMD_TEMP_T_H); <-- Last
				success = Sens_Humi_Write_Cmd(SENSOR_HUMID_H);
				cmDelayMs(5);
				break;
				}

		case 1:
				{
				// Read and store humidity value
				if (success)
					{
					//Sensor_Select(HUMIDITY);
					success = Humi_Read_Data(buf + DATA_LEN, DATA_LEN);
					//Sens_Humi_Write_Cmd(SENSOR_HUMID_RESET);
					//cmDelayMs(15);
					}

				break;
				}
		}

	return success;
	}

/**************************************************************************************************
* @fn          PresTempExecMeasurementStep
*
* @brief       Execute measurement step
*
* @return      none
*/
bool PressureMeasurementStep(void)
	{
	Sensor_Select(PRES_TEMP);

	success = Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_RESET);
	cmDelayMs(20);

	// Start Pressure read
	success = Sens_Humi_Write_Cmd(SENSOR_PRES_256);
	cmDelayMs(15);// 17 ms max for highest resolution

	// start temp read
	if (success)
		{
		success = Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_ADC_READ);
		cmDelayMs(15);
		}

	// Read and store humidity value
	if (success)
		{
		success = Sens_Read_Data(pres_buf, PRES_TEMP_LEN);
		//Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_RESET);
		cmDelayMs(15);
		}

	return success;
	}

/**************************************************************************************************
* @fn          TempMeasurementStep
*
* @brief       Execute measurement step
*
* @return      none
*/
bool TempMeasurementStep()
	{
	Sensor_Select(PRES_TEMP);
	//cmDelayMs(10);

	//success = Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_RESET);
	// Start Pressure read
	success = Sens_Humi_Write_Cmd(SENSOR_TEMP_256);
	cmDelayMs(15);

	// start temp read
	if (success)
		{
		success = Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_ADC_READ);
		cmDelayMs(15);
		}

	// Read and store humidity value
	if (success)
		{
		success = Sens_Read_Data(temp_buf, PRES_TEMP_LEN);
		cmDelayMs(15);
		//Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_RESET);
		//cmDelayMs(15);
		}


	return success;
	}

/**************************************************************************************************
* @fn          HalHumiReadMeasurement
*
* @brief       Get humidity sensor data
*
* @return      none
*/
bool HumiReadMeasurement(UInt8 *pBuf)
	{
	// Store temperature
	pBuf[0] = buf[1];
	pBuf[1] = buf[0];

	// Store humidity
	pBuf[2] = buf[4];
	pBuf[3] = buf[3];

	return success;
	}

bool PresReadMeasurement(unsigned long *pBuf)
	{
	unsigned long tmp = 0;
	unsigned long MSB = 0;
	unsigned long TSB = 0;
	unsigned long LSB = 0;
	memcpy(&MSB, &pres_buf[0], sizeof(MSB));
	tmp += MSB << 16;

	memcpy(&TSB, &pres_buf[1], sizeof(TSB));
	tmp += TSB << 8;

	memcpy(&LSB, &pres_buf[2], sizeof(LSB));
	tmp += LSB;

	memcpy(&raw_pres, &tmp, sizeof(raw_pres));

	//printf("Raw pressure = %d \n", tmp);

	return 1;
	}

bool TempReadMeasurement(unsigned long *pBuf)
	{
	unsigned long temp = 0;
	unsigned long MSB = 0;
	unsigned long TSB = 0;
	unsigned long LSB = 0;
	memcpy(&MSB, &temp_buf[0], sizeof(MSB));
	temp += MSB << 16;

	memcpy(&TSB, &temp_buf[1], sizeof(TSB));
	temp += TSB << 8;

	memcpy(&LSB, &temp_buf[2], sizeof(LSB));
	temp += LSB;

	memcpy(&raw_temp, &temp, sizeof(raw_temp));

// printf("Raw temp = %d \n", temp);

	return 1;
	}

/*
  SENSOR _ PROM
*/
void Sensor_Prom(char coef_num)
	{
	if (coef_num == 0)
		{
		HalI2CDisable();
		Sensor_Select(PRES_TEMP);
		Sens_Humi_Write_Cmd(SENSOR_PRES_TEMP_RESET);
		cmDelayMs(10);
		}

	success = Sens_Humi_Write_Cmd(SENSOR_PROM_READ + coef_num * 2);
	cmDelayMs(10);

	success = Sens_Read_Data(&C[coef_num], 2);

	C[coef_num] = C[coef_num] * 256;
	}


/* ------------------------------------------------------------------------------------------------
*                                           Private functions
* -------------------------------------------------------------------------------------------------
*/

/**************************************************************************************************
* @fn          Sensor_Select
*
* @brief       Select the humidity sensor on the I2C-bus
*
* @return
*/
void Sensor_Select(int sensor)
	{
	HalI2CDisable();
	switch (sensor)
		{
		case HUMIDITY:
				{
				selectI2C(SENSOR_HUMIDITY_I2C_ADDRESS, i2cClock_267KHZ);
				break;
				}

		case PRES_TEMP:
				{
				selectI2C(SENSOR_PRES_TEMP_I2C_ADDRESS, i2cClock_267KHZ);
				break;
				}
		}
	}


/**************************************************************************************************
* @fn          Sens_Humi_Write_Cmd
*
* @brief       Write a command to the humidity sensor
*
* @param       cmd - command to write
*
* @return      TRUE if the command has been transmitted successfully
**************************************************************************************************/
static bool Sens_Humi_Write_Cmd(UInt8 cmd)
	{
	/* Send command */
	return HalI2CWrite(1, &cmd) == 1;
	}


/**************************************************************************************************
* @fn          HalHumiReadData
*
* @brief       This function implements the I2C protocol to read from the HTU21D.
*
* @param       pBuf - pointer to buffer to place data
*
* @param       nBytes - number of bytes to read
*
* @return      TRUE if the required number of bytes are received
**************************************************************************************************/
static bool Humi_Read_Data(unsigned long *pBuf, UInt8 nBytes)
	{
	/* Read data */
	return HalI2CRead(nBytes, pBuf) == nBytes;
	}

static bool Sens_Read_Data(unsigned long *pBuf, UInt8 nBytes)
	{
	/* Read data */
	return HalI2CRead(nBytes, pBuf) == nBytes;
	}



//  Conversion algorithm, humidity
double calcHumRel(UInt16 rawH)
	{
	rawH &= ~0x0003; // clear bits [1..0] (status bits)
	//-- calculate relative humidity [%RH] --
	return (-7.0 + 125.0 / 65536 * (double)rawH) * 10; // RH= -6 + 125 * SRH/2^16
	}

//  Conversion algorithm, temperature
double *calcTempPress()
	{
	static double data[2];

	dT = raw_temp - (C[5] * pow(2, 8));
	//T = (2000 + (dT * C[6]) / pow(2, 23)) / 100;
	T = (2000 + (dT * C[6]) / pow(2, 23)) / 1;

	OFF = C[2] * pow(2, 17) + dT * C[4] / pow(2, 6);
	SENSE = C[1] * pow(2, 16) + dT * C[3] / pow(2, 7);
	Pressure = (((raw_pres * SENSE) / pow(2, 21) - OFF) / pow(2, 15)) / 100;

	memcpy(&data[0], &T, sizeof(T));
	memcpy(&data[1], &Pressure, sizeof(Pressure));

	return data;
	}


double *calcPres() 
	{
	  static double data[2];
	OFF = C[2] * pow(2, 17) + (dT * C[4]) / pow(2, 6);
	SENSE = C[1] * pow(2, 16) + (dT * C[3]) / pow(2, 7);
	Pressure = (((raw_pres * SENSE) / pow(2, 21) - OFF) / pow(2, 15)) / 100;

	memcpy(&data[0], &Pressure, sizeof(Pressure));
	
	return data; //JON CHANGE
	}


/**************************************************************************************************
 * @fn          SensorReadReg
 *
 * @brief       This function implements the I2C protocol to read from a sensor. The sensor must
 *              be selected before this routine is called.
 *
 * @param       addr - which register to read
 * @param       pBuf - pointer to buffer to place data
 * @param       nBytes - numbver of bytes to read
 *
 * @return      TRUE if the required number of bytes are reveived
 **************************************************************************************************/
bool SensorReadReg(UInt8 addr, unsigned long *pBuf, UInt8 nBytes)
	{
	UInt8 i = 0;

	/* Send address we're reading from */
	if (HalI2CWrite(1, &addr) == 1)
		{
		/* Now read data */
		i = HalI2CRead(nBytes, pBuf);
		}

	return i == nBytes;
	}

/**************************************************************************************************
* @fn          SensorWriteReg
* @brief       This function implements the I2C protocol to write to a sensor. he sensor must
*              be selected before this routine is called.
*
* @param       addr - which register to write
* @param       pBuf - pointer to buffer containing data to be written
* @param       nBytes - number of bytes to write
*
* @return      TRUE if successful write
*/
bool SensorWriteReg(UInt8 addr, unsigned long *pBuf, UInt8 nBytes)
	{
	UInt8 i;
	UInt8 *p = buffer;

	/* Copy address and data to local buffer for burst write */
	*p++ = addr;
	for (i = 0; i < nBytes; i++)
		{
		*p++ = *pBuf++;
		}
	nBytes++;

	/* Send address and data */
	i = HalI2CWrite(nBytes, buffer);
	if (i != nBytes)
		{
		// HAL_TOGGLE_LED2();
		}
	return (i == nBytes);
	}
