#include "EtherDATA.h"
#include "kNode.ddl.h"
#include "adc.h"
#include "sysGpio.h"
#include "led.h"
#include "string.h"
#include "pindefs.h"

#define BAT_SAMPLES             3
#define OUTLIER_THRESHOLD       5

int printf(const char *, ...);
void MemledOn(word led, word delay);
word adc(byte spec);

UInt16 bat_timer_counter; // variable that counts # of times the handler is called
UInt16 bat_sample_counter;
UInt16 avg_value = 0;
UInt16 sample_set[BAT_SAMPLES];

DefineTimer(lowBatLEDTimer);
void lowBatLED(TIMER t)
	{
	MemledOn(LED_RED, 100);
	//ledBlink(redFlashPattern);
	}

UInt16 batterySample()
	{
	UInt16 value = 0;

	GPIO.SET(BATT_EN, 1);

	cmDelayMs(50); // allow circuitry to stabalize

	value = (adc(0x00 | 0x30 | AIN0) >> 3); // VDD/3 ref. AIN0 as source

	GPIO.SET(BATT_EN, 0); // turn off circuitry

	return value;
	}

// Calculation is based off ADC measurements. Data in EXCEL spreadsheet under kitchen battery (****subject to change, adjustments needed***)
UInt16 batteryCalc(UInt16 adc)
	{
	UInt16 percent = 0;

	if (adc >= 3595)
		{
		if (adc > 3600)
			{
			percent = 100;
			}
		else
			{
			percent = (adc - 3595) + 95;
			}
		}
	else
		{
		if (adc < 602)
			{
			percent = 1;
			}
		else
			{
			percent = ((adc - 602) * 6) / 187;
			//percent = ((adc - 1520) * 25 +563) / 564;
			}
		//printf("percent: %u\n", percent);
		}

	return percent;
	}

// define a battery sample timer
DefineTimer(batterySampleTimer);
void batterySampleTimerHandler(TIMER e, dbValue ref, UInt16 time)
	{
	bat_timer_counter ++;
	if (bat_timer_counter >= (29 * time))//sample every 29 min
		{
		word value = 0;
		bat_timer_counter = 0;
		value = batterySample();
		//printf("Bat ADC Value = %d \n", value); // debug statement
		avg_value += value; // add value to average variable
		bat_sample_counter ++;
		}

	if (bat_sample_counter >= (1)) // if its been 'x' samples, calc avg and publish
		{
		bat_sample_counter = 0;
		UInt16 percent = 0;
		avg_value = avg_value / 2; // divide sum by 'x' number of samples
		printf("BAT AVG = %d \n", avg_value);
		percent = batteryCalc(avg_value);
		avg_value = 0;

		dbSet(ref, percent); // publish percent
		bat_sample_counter = 0;

		printf("bat:%d % \n", percent); // debug statement
		}


	}

void initBattery() // init timers, start sample timer, and publish batteryValue
	{
	bat_timer_counter = 0;
	// set up and start the battery sampling timer
	objectCreate(batterySampleTimer, kIntervalTimer, TICKS(30000));
	OnEvent(batterySampleTimer, (HANDLER) batterySampleTimerHandler);

	objectCreate(lowBatLEDTimer, kIntervalTimer, TICKS(1000));
	OnEvent(lowBatLEDTimer, (HANDLER) lowBatLED);

	cmStartTimer(batterySampleTimer, batteryValue, 2);

	dbPublish(batteryValue);
	}
