#include "EtherDATA.h"
#include "LIS2DH.h"
#include "kNode.ddl.h"
#include "adc.h"
#include "sysGpio.h"
#include "buzzer.h"
#include "led.h"
#include "pindefs.h"
#include "string.h"
#include "TCA6416A.h"

#define kLEDBlinkDelay 250

int printf(const char *, ...);
void MemledOn(word led, word delay);
void TopledOn(word delay);
void configDrop();
void configDoor();
byte toggle_count = 0;
ExternTimer(findTagLEDTimer);
ExternTimer(configDeviceLEDTimer);
ExternTimer(awakeTimer);
ExternTimer(dataLogTimer);
extern UInt16 dataLog_counter;
UInt16 sample_number;
UInt16 kitchenStatus_counter = 0;
byte kitchen_mode = 0;
byte blink_counter = 0;

DefineTimer(buttonDataTimer);
void buttonDataTimerHandler(TIMER e)
	{
	toggle_count = 0;
	dbPublish(buttonData);
	}

void buttonHeld(int button) // User interface when button is held
	{
	switch (button)
		{
		case BUTTON_1: // set device type, stop configLED pattern if its running
			printf("b1.held\n");
			cmStopTimer(configDeviceLEDTimer);
			dbSet(deviceType, 4);
			TopledOn(100);
			break;

		case BUTTON_2: // no logic set for now
			printf("b2.held\n");
			break;

		case IOX_INT: // no logic set for now
			printf("b3.held\n");
			break;

		case 1: //both 1 and 3 held
			printf("b1&b3.held\n");
			if (!(kitchen_mode == 4))
				{
				dbSet(buttonData, 4); // set buttonData to 4
				kitchen_mode = 4; // kitchen mode is 4
				TopledOn(100);
				}
			else // toggle kitchen mode from 4 to 8
				{
				dbSet(buttonData, 8);
				playTune(msgTune);
				kitchen_mode = 8;
				}
			break;
		}
	}

void buttonPressed(int button)
	{
	cmStopTimer(findTagLEDTimer);
	GPIO.SET(BUZZER, 0);
	switch (button)
		{
		case BUTTON_1:
			printf("b1\n");
			dbSet(buttonData, 1);
			kitchen_mode = 1;
			break;

		case BUTTON_2:
			printf("b2\n");
			dbSet(buttonData, 2);
			kitchen_mode = 2;
			break;

		case IOX_INT:
			printf("b3\n");
			dbSet(buttonData, 3);
			kitchen_mode = 3;
			break;
		}

	if (!toggle_count) // limit the amount of buttonData publishes (should only be AT MOST one per second interval)
		{
		toggle_count = 1;
		dbWithdraw(buttonData);
		cmStartTimer(buttonDataTimer);
		}
	}

// define a accel sample timer
DefineTimer(kitchenBlinkTimer);
void kitchenBlinkTimerHandler(TIMER e)
	{
	MemledOn(LED_GRN, 100);
	blink_counter --;
	if (!blink_counter)
		{
		cmStopTimer(kitchenBlinkTimer);
		}
	}

// define a kitchen timer
DefineTimer(kitchenStatusTimer); // kitchen timer simply blinks green led depending on what kitchen mode it is in
void kitchenStatusTimerHandler(TIMER e)
	{
	kitchenStatus_counter ++;
	if (kitchenStatus_counter >= 1)
		{
		kitchenStatus_counter = 0;
		switch (kitchen_mode)
			{
			case 1:
					{
					blink_counter = 1;
					cmStartTimer(kitchenBlinkTimer);
					/*  for(int x = 0; x < 1; x++)
					  {
					  MemledOn(LED_GRN,100);
					  //TopledOn(100);
					  cmDelayMs(500);
					  }*/
					break;
					}
			case 2:
					{
					blink_counter = 2;
					cmStartTimer(kitchenBlinkTimer);
					/*  for(int x = 0; x < 2; x++)
					  {
					  MemledOn(LED_GRN,100);
					  //TopledOn(100);
					  cmDelayMs(500);
					  }*/
					break;
					}
			case 3:
					{
					blink_counter = 3;
					cmStartTimer(kitchenBlinkTimer);
					/*  for(int x = 0; x < 3; x++)
					  {
					  MemledOn(LED_GRN,100);
					  //TopledOn(100);
					  cmDelayMs(500);
					  }*/
					break;
					}
			case 4:
					{
					TopledOn(100);
					break;
					}
			default:
					{}
			}
		printf("Mode=%d\n", kitchen_mode);
		}
	}

void initUI() // sets deviceType and buttonData to publish. Creates timers and starts kitchen status timer
	{
	dbPublish(deviceType);
	dbPublish(buttonData);

	device_state = TRACKER_MODE;
	dbSet(deviceType, 4);

	// set up and start the accelerometer sampling timer
	objectCreate(kitchenStatusTimer, kIntervalTimer, TICKS(30000));
	OnEvent(kitchenStatusTimer, (HANDLER) kitchenStatusTimerHandler);
	cmStartTimer(kitchenStatusTimer);

	// set up and start the accelerometer sampling timer
	objectCreate(kitchenBlinkTimer, kIntervalTimer, TICKS(1000));
	OnEvent(kitchenBlinkTimer, (HANDLER) kitchenBlinkTimerHandler);

	// set up the button Data limit timer (limits the button data publish once per sync interval)
	objectCreate(buttonDataTimer, kSingleShotTimer, TICKS(5000));
	OnEvent(buttonDataTimer, (HANDLER) buttonDataTimerHandler);
	//printf("kitchen timer started \n");
	}
